@@ -32,10 +32,10 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 // for getline(3)
 #define _POSIX_C_SOURCE 200809L
 
-#include <sys/stat.h>
 #include <dirent.h>
-#include <unistd.h>
 #include <inttypes.h>
+#include <sys/stat.h>
+#include <unistd.h>
 
 #else
 
@@ -46,37 +46,34 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #endif
 
-#include <time.h>
+#include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <ctype.h>
-
+#include <time.h>
 #include <yara.h>
 
 #include "args.h"
 #include "common.h"
 #include "threading.h"
 
-
-#define ERROR_COULD_NOT_CREATE_THREAD  100
+#define ERROR_COULD_NOT_CREATE_THREAD 100
 
 #ifndef MAX_PATH
 #define MAX_PATH 256
 #endif
 
 #ifndef min
-#define min(x, y)  ((x < y) ? (x) : (y))
+#define min(x, y) ((x < y) ? (x) : (y))
 #endif
 
 #ifdef _MSC_VER
 #define snprintf _snprintf
-#define strdup _strdup
+#define strdup   _strdup
 #endif
 
 #define MAX_QUEUED_FILES 64
 
-
 typedef struct _MODULE_DATA
 {
   const char* module_name;
@@ -85,7 +82,6 @@ typedef struct _MODULE_DATA
 
 } MODULE_DATA;
 
-
 typedef struct _CALLBACK_ARGS
 {
   const char* file_path;
@@ -93,24 +89,21 @@ typedef struct _CALLBACK_ARGS
 
 } CALLBACK_ARGS;
 
-
 typedef struct _THREAD_ARGS
 {
-  YR_SCANNER*       scanner;
-  CALLBACK_ARGS     callback_args;
-  time_t            start_time;
-  int               current_count;
+  YR_SCANNER* scanner;
+  CALLBACK_ARGS callback_args;
+  time_t deadline;
+  int current_count;
 
 } THREAD_ARGS;
 
-
 typedef struct _QUEUED_FILE
 {
   char* path;
 
 } QUEUED_FILE;
 
-
 typedef struct COMPILER_RESULTS
 {
   int errors;
@@ -118,11 +111,18 @@ typedef struct COMPILER_RESULTS
 
 } COMPILER_RESULTS;
 
+typedef struct SCAN_OPTIONS
+{
+  bool follow_symlinks;
+  bool recursive_search;
+  time_t deadline;
 
-#define MAX_ARGS_TAG            32
-#define MAX_ARGS_IDENTIFIER     32
-#define MAX_ARGS_EXT_VAR        32
-#define MAX_ARGS_MODULE_DATA    32
+} SCAN_OPTIONS;
+
+#define MAX_ARGS_TAG         32
+#define MAX_ARGS_IDENTIFIER  32
+#define MAX_ARGS_EXT_VAR     32
+#define MAX_ARGS_MODULE_DATA 32
 
 static char* atom_quality_table;
 static char* tags[MAX_ARGS_TAG + 1];
@@ -130,6 +130,7 @@ static char* identifiers[MAX_ARGS_IDENTIFIER + 1];
 static char* ext_vars[MAX_ARGS_EXT_VAR + 1];
 static char* modules_data[MAX_ARGS_MODULE_DATA + 1];
 
+static bool follow_symlinks = true;
 static bool recursive_search = false;
 static bool scan_list_search = false;
 static bool show_module_data = false;
@@ -154,97 +155,158 @@ static int stack_size = DEFAULT_STACK_SIZE;
 static int threads = YR_MAX_THREADS;
 static int max_strings_per_rule = DEFAULT_MAX_STRINGS_PER_RULE;
 
-
 #define USAGE_STRING \
-    "Usage: yara [OPTION]... [NAMESPACE:]RULES_FILE... FILE | DIR | PID"
+  "Usage: yara [OPTION]... [NAMESPACE:]RULES_FILE... FILE | DIR | PID"
 
+args_option_t options[] = {
+    OPT_STRING(
+        0,
+        "atom-quality-table",
+        &atom_quality_table,
+        "path to a file with the atom quality table",
+        "FILE"),
 
-args_option_t options[] =
-{
-  OPT_STRING(0, "atom-quality-table", &atom_quality_table,
-      "path to a file with the atom quality table", "FILE"),
+    OPT_BOOLEAN(
+        'C',
+        "compiled-rules",
+        &rules_are_compiled,
+        "load compiled rules"),
 
-  OPT_BOOLEAN('C', "compiled-rules", &rules_are_compiled,
-      "load compiled rules"),
+    OPT_BOOLEAN(
+        'c',
+        "count",
+        &print_count_only,
+        "print only number of matches"),
 
-  OPT_BOOLEAN('c', "count", &print_count_only,
-      "print only number of matches"),
+    OPT_STRING_MULTI(
+        'd',
+        "define",
+        &ext_vars,
+        MAX_ARGS_EXT_VAR,
+        "define external variable",
+        "VAR=VALUE"),
 
-  OPT_STRING_MULTI('d', "define", &ext_vars, MAX_ARGS_EXT_VAR,
-      "define external variable", "VAR=VALUE"),
+    OPT_BOOLEAN(0, "fail-on-warnings", &fail_on_warnings, "fail on warnings"),
 
-  OPT_BOOLEAN(0, "fail-on-warnings", &fail_on_warnings,
-      "fail on warnings"),
+    OPT_BOOLEAN('f', "fast-scan", &fast_scan, "fast matching mode"),
 
-  OPT_BOOLEAN('f', "fast-scan", &fast_scan,
-      "fast matching mode"),
+    OPT_BOOLEAN('h', "help", &show_help, "show this help and exit"),
 
-  OPT_BOOLEAN('h', "help", &show_help,
-      "show this help and exit"),
+    OPT_STRING_MULTI(
+        'i',
+        "identifier",
+        &identifiers,
+        MAX_ARGS_IDENTIFIER,
+        "print only rules named IDENTIFIER",
+        "IDENTIFIER"),
 
-  OPT_STRING_MULTI('i', "identifier", &identifiers, MAX_ARGS_IDENTIFIER,
-      "print only rules named IDENTIFIER", "IDENTIFIER"),
+    OPT_INTEGER(
+        'l',
+        "max-rules",
+        &limit,
+        "abort scanning after matching a NUMBER of rules",
+        "NUMBER"),
 
-  OPT_INTEGER('l', "max-rules", &limit,
-      "abort scanning after matching a NUMBER of rules", "NUMBER"),
+    OPT_INTEGER(
+        0,
+        "max-strings-per-rule",
+        &max_strings_per_rule,
+        "set maximum number of strings per rule (default=10000)",
+        "NUMBER"),
 
-  OPT_INTEGER(0, "max-strings-per-rule", &max_strings_per_rule,
-      "set maximum number of strings per rule (default=10000)", "NUMBER"),
+    OPT_STRING_MULTI(
+        'x',
+        "module-data",
+        &modules_data,
+        MAX_ARGS_MODULE_DATA,
+        "pass FILE's content as extra data to MODULE",
+        "MODULE=FILE"),
 
-  OPT_STRING_MULTI('x', "module-data", &modules_data, MAX_ARGS_MODULE_DATA,
-      "pass FILE's content as extra data to MODULE", "MODULE=FILE"),
+    OPT_BOOLEAN(
+        'n',
+        "negate",
+        &negate,
+        "print only not satisfied rules (negate)",
+        NULL),
 
-  OPT_BOOLEAN('n', "negate", &negate,
-      "print only not satisfied rules (negate)", NULL),
+    OPT_BOOLEAN('w', "no-warnings", &ignore_warnings, "disable warnings"),
 
-  OPT_BOOLEAN('w', "no-warnings", &ignore_warnings,
-      "disable warnings"),
+    OPT_BOOLEAN('m', "print-meta", &show_meta, "print metadata"),
 
-  OPT_BOOLEAN('m', "print-meta", &show_meta,
-      "print metadata"),
+    OPT_BOOLEAN(
+        'D',
+        "print-module-data",
+        &show_module_data,
+        "print module data"),
 
-  OPT_BOOLEAN('D', "print-module-data", &show_module_data,
-      "print module data"),
+    OPT_BOOLEAN(
+        'e',
+        "print-namespace",
+        &show_namespace,
+        "print rules' namespace"),
 
-  OPT_BOOLEAN('e', "print-namespace", &show_namespace,
-      "print rules' namespace"),
+    OPT_BOOLEAN('S', "print-stats", &show_stats, "print rules' statistics"),
 
-  OPT_BOOLEAN('S', "print-stats", &show_stats,
-      "print rules' statistics"),
+    OPT_BOOLEAN('s', "print-strings", &show_strings, "print matching strings"),
 
-  OPT_BOOLEAN('s', "print-strings", &show_strings,
-      "print matching strings"),
+    OPT_BOOLEAN(
+        'L',
+        "print-string-length",
+        &show_string_length,
+        "print length of matched strings"),
 
-  OPT_BOOLEAN('L', "print-string-length", &show_string_length,
-      "print length of matched strings"),
+    OPT_BOOLEAN('g', "print-tags", &show_tags, "print tags"),
 
-  OPT_BOOLEAN('g', "print-tags", &show_tags,
-      "print tags"),
+    OPT_BOOLEAN(
+        'r',
+        "recursive",
+        &recursive_search,
+        "recursively search directories"),
 
-  OPT_BOOLEAN('r', "recursive", &recursive_search,
-      "recursively search directories (follows symlinks)"),
+    OPT_BOOLEAN(
+        'N',
+        "no-follow-symlinks",
+        &follow_symlinks,
+        "do not follow symlinks when scanning"),
 
-  OPT_BOOLEAN(0, "scan-list", &scan_list_search,
-      "scan files listed in FILE, one per line"),
+    OPT_BOOLEAN(
+        0,
+        "scan-list",
+        &scan_list_search,
+        "scan files listed in FILE, one per line"),
 
-  OPT_INTEGER('k', "stack-size", &stack_size,
-      "set maximum stack size (default=16384)", "SLOTS"),
+    OPT_INTEGER(
+        'k',
+        "stack-size",
+        &stack_size,
+        "set maximum stack size (default=16384)",
+        "SLOTS"),
 
-  OPT_STRING_MULTI('t', "tag", &tags, MAX_ARGS_TAG,
-      "print only rules tagged as TAG", "TAG"),
+    OPT_STRING_MULTI(
+        't',
+        "tag",
+        &tags,
+        MAX_ARGS_TAG,
+        "print only rules tagged as TAG",
+        "TAG"),
 
-  OPT_INTEGER('p', "threads", &threads,
-      "use the specified NUMBER of threads to scan a directory", "NUMBER"),
+    OPT_INTEGER(
+        'p',
+        "threads",
+        &threads,
+        "use the specified NUMBER of threads to scan a directory",
+        "NUMBER"),
 
-  OPT_INTEGER('a', "timeout", &timeout,
-      "abort scanning after the given number of SECONDS", "SECONDS"),
+    OPT_INTEGER(
+        'a',
+        "timeout",
+        &timeout,
+        "abort scanning after the given number of SECONDS",
+        "SECONDS"),
 
-  OPT_BOOLEAN('v', "version", &show_version,
-      "show version information"),
-
-  OPT_END()
-};
+    OPT_BOOLEAN('v', "version", &show_version, "show version information"),
 
+    OPT_END()};
 
 // file_queue is size-limited queue stored as a circular array, files are
 // removed from queue_head position and new files are added at queue_tail
@@ -265,7 +327,6 @@ MUTEX output_mutex;
 
 MODULE_DATA* modules_data_list = NULL;
 
-
 static int file_queue_init()
 {
   int result;
@@ -273,59 +334,57 @@ static int file_queue_init()
   queue_tail = 0;
   queue_head = 0;
 
-  result = mutex_init(&queue_mutex);
+  result = cli_mutex_init(&queue_mutex);
 
   if (result != 0)
     return result;
 
-  result = semaphore_init(&used_slots, 0);
+  result = cli_semaphore_init(&used_slots, 0);
 
   if (result != 0)
     return result;
 
- return semaphore_init(&unused_slots, MAX_QUEUED_FILES);
+  return cli_semaphore_init(&unused_slots, MAX_QUEUED_FILES);
 }
 
-
 static void file_queue_destroy()
 {
-  mutex_destroy(&queue_mutex);
-  semaphore_destroy(&unused_slots);
-  semaphore_destroy(&used_slots);
+  cli_mutex_destroy(&queue_mutex);
+  cli_semaphore_destroy(&unused_slots);
+  cli_semaphore_destroy(&used_slots);
 }
 
-
 static void file_queue_finish()
 {
-  int i;
-
-  for (i = 0; i < YR_MAX_THREADS; i++)
-    semaphore_release(&used_slots);
+  for (int i = 0; i < YR_MAX_THREADS; i++) cli_semaphore_release(&used_slots);
 }
 
-
-static void file_queue_put(
-    const char* file_path)
+static int file_queue_put(const char* file_path, time_t deadline)
 {
-  semaphore_wait(&unused_slots);
-  mutex_lock(&queue_mutex);
+  if (cli_semaphore_wait(&unused_slots, deadline) == ERROR_SCAN_TIMEOUT)
+    return ERROR_SCAN_TIMEOUT;
+
+  cli_mutex_lock(&queue_mutex);
 
   file_queue[queue_tail].path = strdup(file_path);
   queue_tail = (queue_tail + 1) % (MAX_QUEUED_FILES + 1);
 
-  mutex_unlock(&queue_mutex);
-  semaphore_release(&used_slots);
+  cli_mutex_unlock(&queue_mutex);
+  cli_semaphore_release(&used_slots);
+
+  return ERROR_SUCCESS;
 }
 
-
-static char* file_queue_get()
+static char* file_queue_get(time_t deadline)
 {
   char* result;
 
-  semaphore_wait(&used_slots);
-  mutex_lock(&queue_mutex);
+  if (cli_semaphore_wait(&used_slots, deadline) == ERROR_SCAN_TIMEOUT)
+    return NULL;
 
-  if (queue_head == queue_tail) // queue is empty
+  cli_mutex_lock(&queue_mutex);
+
+  if (queue_head == queue_tail)  // queue is empty
   {
     result = NULL;
   }
@@ -335,32 +394,28 @@ static char* file_queue_get()
     queue_head = (queue_head + 1) % (MAX_QUEUED_FILES + 1);
   }
 
-  mutex_unlock(&queue_mutex);
-  semaphore_release(&unused_slots);
+  cli_mutex_unlock(&queue_mutex);
+  cli_semaphore_release(&unused_slots);
 
   return result;
 }
 
-
 #if defined(_WIN32) || defined(__CYGWIN__)
 
-static bool is_directory(
-    const char* path)
+static bool is_directory(const char* path)
 {
   DWORD attributes = GetFileAttributes(path);
 
   if (attributes != INVALID_FILE_ATTRIBUTES &&
-	  attributes & FILE_ATTRIBUTE_DIRECTORY)
+      attributes & FILE_ATTRIBUTE_DIRECTORY)
     return true;
   else
     return false;
 }
 
-static void scan_dir(
-    const char* dir,
-    int recursive,
-    time_t start_time)
+static int scan_dir(const char* dir, SCAN_OPTIONS* scan_opts)
 {
+  int result = ERROR_SUCCESS;
   static char path_and_mask[MAX_PATH];
 
   snprintf(path_and_mask, sizeof(path_and_mask), "%s\\*", dir);
@@ -374,42 +429,43 @@ static void scan_dir(
     {
       char full_path[MAX_PATH];
 
-      snprintf(full_path, sizeof(full_path), "%s\\%s",
-               dir, FindFileData.cFileName);
+      snprintf(
+          full_path, sizeof(full_path), "%s\\%s", dir, FindFileData.cFileName);
 
       if (!(FindFileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
       {
-        file_queue_put(full_path);
+        result = file_queue_put(full_path, scan_opts->deadline);
       }
-      else if (recursive &&
-               strcmp(FindFileData.cFileName, ".") != 0 &&
-               strcmp(FindFileData.cFileName, "..") != 0)
+      else if (
+          scan_opts->recursive_search &&
+          strcmp(FindFileData.cFileName, ".") != 0 &&
+          strcmp(FindFileData.cFileName, "..") != 0)
       {
-        scan_dir(full_path, recursive, start_time);
+        result = scan_dir(full_path, scan_opts);
       }
 
-    } while (FindNextFile(hFind, &FindFileData));
+    } while (result != ERROR_SCAN_TIMEOUT &&
+             FindNextFile(hFind, &FindFileData));
 
     FindClose(hFind);
   }
+
+  return result;
 }
 
-
-#if defined(__CYGWIN__)
-#define strtok_s strtok_r
-#endif
-
-static int populate_scan_list(
-    const char* filename,
-    int recursive,
-    time_t start_time)
+static int populate_scan_list(const char* filename, SCAN_OPTIONS* scan_opts)
 {
-  char* context;
   DWORD nread;
+  int result = ERROR_SUCCESS;
 
   HANDLE hFile = CreateFile(
-      filename, GENERIC_READ, FILE_SHARE_READ, NULL,
-      OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
+      filename,
+      GENERIC_READ,
+      FILE_SHARE_READ,
+      NULL,
+      OPEN_EXISTING,
+      FILE_ATTRIBUTE_NORMAL,
+      NULL);
 
   if (hFile == INVALID_HANDLE_VALUE)
   {
@@ -421,8 +477,8 @@ static int populate_scan_list(
 
   if (fileSize == INVALID_FILE_SIZE)
   {
-    fprintf(stderr,
-      "error: could not determine size of file \"%s\".\n", filename);
+    fprintf(
+        stderr, "error: could not determine size of file \"%s\".\n", filename);
     CloseHandle(hFile);
     return ERROR_COULD_NOT_READ_FILE;
   }
@@ -433,8 +489,10 @@ static int populate_scan_list(
 
   if (buf == NULL)
   {
-    fprintf(stderr,
-        "error: could not allocate memory for file \"%s\".\n", filename);
+    fprintf(
+        stderr,
+        "error: could not allocate memory for file \"%s\".\n",
+        filename);
     CloseHandle(hFile);
     return ERROR_INSUFFICIENT_MEMORY;
   }
@@ -452,11 +510,13 @@ static int populate_scan_list(
     total += nread;
   }
 
-  char* path = strtok_s(buf, "\n", &context);
+  // Note: There's no need for reentrant strtok variants since this
+  // function is run in single-threaded code.
+  char* path = strtok(buf, "\n");
 
-  while (path != NULL)
+  while (result != ERROR_SCAN_TIMEOUT && path != NULL)
   {
-    // remove trailing carriage return, if present
+    // Remove trailing carriage return, if present.
     if (*path != '\0')
     {
       char* final = path + strlen(path) - 1;
@@ -464,62 +524,84 @@ static int populate_scan_list(
         *final = '\0';
     }
     if (is_directory(path))
-      scan_dir(path, recursive, start_time);
+      result = scan_dir(path, scan_opts);
     else
-      file_queue_put(path);
-    path = strtok_s(NULL, "\n", &context);
+      result = file_queue_put(path, scan_opts->deadline);
+
+    path = strtok(NULL, "\n");
   }
 
   CloseHandle(hFile);
-  return ERROR_SUCCESS;
+
+  return result;
 }
 
 #else
 
-static bool is_directory(
-    const char* path)
+static bool is_directory(const char* path)
 {
   struct stat st;
 
-  if (stat(path,&st) == 0)
+  if (stat(path, &st) == 0)
     return S_ISDIR(st.st_mode);
 
   return 0;
 }
 
-
-static void scan_dir(
-    const char* dir,
-    int recursive,
-    time_t start_time)
+static int scan_dir(const char* dir, SCAN_OPTIONS* scan_opts)
 {
+  int result = ERROR_SUCCESS;
   DIR* dp = opendir(dir);
 
   if (dp)
   {
     struct dirent* de = readdir(dp);
 
-    while (de && difftime(time(NULL), start_time) < timeout)
+    while (de && result != ERROR_SCAN_TIMEOUT)
     {
       char full_path[MAX_PATH];
       struct stat st;
 
       snprintf(full_path, sizeof(full_path), "%s/%s", dir, de->d_name);
 
-      int err = stat(full_path, &st);
+      int err = lstat(full_path, &st);
+
+      // If lstat returned error, or this directory entry is a symlink and the
+      // user doesn't want to follow symlinks, skip the entry and continue with
+      // the next one.
+      if (err != 0 || (S_ISLNK(st.st_mode) && !scan_opts->follow_symlinks))
+      {
+        de = readdir(dp);
+        continue;
+      }
+      // If the directory entry is a symlink, check if it points to . and
+      // skip it in that case.
+      else if (S_ISLNK(st.st_mode))
+      {
+        char buf[2];
+
+        int len = readlink(full_path, buf, sizeof(buf));
+
+        if (len == 1 && strcmp(buf, ".") == 0)
+        {
+          de = readdir(dp);
+          continue;
+        }
+      }
+
+      err = stat(full_path, &st);
 
       if (err == 0)
       {
-        if(S_ISREG(st.st_mode))
+        if (S_ISREG(st.st_mode))
         {
-          file_queue_put(full_path);
+          result = file_queue_put(full_path, scan_opts->deadline);
         }
-        else if(recursive &&
-                S_ISDIR(st.st_mode) &&
-                strcmp(de->d_name, ".") != 0 &&
-                strcmp(de->d_name, "..") != 0)
+        else if (
+            scan_opts->recursive_search && S_ISDIR(st.st_mode) &&
+            strcmp(de->d_name, ".") != 0 && strcmp(de->d_name, "..") != 0)
         {
-          scan_dir(full_path, recursive, start_time);
+          result = scan_dir(full_path, scan_opts);
         }
       }
 
@@ -528,26 +610,27 @@ static void scan_dir(
 
     closedir(dp);
   }
+
+  return result;
 }
 
-
-static int populate_scan_list(
-    const char* filename,
-    int recursive,
-    time_t start_time)
+static int populate_scan_list(const char* filename, SCAN_OPTIONS* scan_opts)
 {
   size_t nsize = 0;
   ssize_t nread;
   char* path = NULL;
+  int result = ERROR_SUCCESS;
 
   FILE* fh_scan_list = fopen(filename, "r");
+
   if (fh_scan_list == NULL)
   {
     fprintf(stderr, "error: could not open file \"%s\".\n", filename);
     return ERROR_COULD_NOT_OPEN_FILE;
   }
 
-  while ((nread = getline(&path, &nsize, fh_scan_list)) != -1)
+  while (result != ERROR_SCAN_TIMEOUT &&
+         (nread = getline(&path, &nsize, fh_scan_list)) != -1)
   {
     // remove trailing newline
     if (nread && path[nread - 1] == '\n')
@@ -555,22 +638,22 @@ static int populate_scan_list(
       path[nread - 1] = '\0';
       nread--;
     }
+
     if (is_directory(path))
-      scan_dir(path, recursive, start_time);
+      result = scan_dir(path, scan_opts);
     else
-      file_queue_put(path);
+      result = file_queue_put(path, scan_opts->deadline);
   }
 
   free(path);
   fclose(fh_scan_list);
-  return ERROR_SUCCESS;
+
+  return result;
 }
 
 #endif
 
-static void print_string(
-    const uint8_t* data,
-    int length)
+static void print_string(const uint8_t* data, int length)
 {
   for (int i = 0; i < length; i++)
   {
@@ -583,104 +666,90 @@ static void print_string(
   printf("\n");
 }
 
-
-static char cescapes[] =
-{
-  0  , 0  , 0  , 0  , 0  , 0  , 0  , 'a',
-  'b', 't', 'n', 'v', 'f', 'r', 0  , 0  ,
-  0  , 0  , 0  , 0  , 0  , 0  , 0  , 0  ,
-  0  , 0  , 0  , 0  , 0  , 0  , 0  , 0  ,
+static char cescapes[] = {
+    0, 0, 0, 0, 0, 0, 0, 'a', 'b', 't', 'n', 'v', 'f', 'r', 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0,   0,   0,   0,   0,   0,   0,   0, 0,
 };
 
-
-static void print_escaped(
-    const uint8_t* data,
-    size_t length)
+static void print_escaped(const uint8_t* data, size_t length)
 {
-  size_t i;
-
-  for (i = 0; i < length; i++)
+  for (size_t i = 0; i < length; i++)
   {
     switch (data[i])
     {
-      case '\"':
-      case '\'':
-      case '\\':
-        printf("\\%c", data[i]);
-        break;
+    case '\"':
+    case '\'':
+    case '\\':
+      printf("\\%c", data[i]);
+      break;
 
-      default:
-        if (data[i] >= 127)
-          printf("\\%03o", data[i]);
-        else if (data[i] >= 32)
-          putchar(data[i]);
-        else if (cescapes[data[i]] != 0)
-          printf("\\%c", cescapes[data[i]]);
-        else
-          printf("\\%03o", data[i]);
+    default:
+      if (data[i] >= 127)
+        printf("\\%03o", data[i]);
+      else if (data[i] >= 32)
+        putchar(data[i]);
+      else if (cescapes[data[i]] != 0)
+        printf("\\%c", cescapes[data[i]]);
+      else
+        printf("\\%03o", data[i]);
     }
   }
 }
 
-
-static void print_hex_string(
-    const uint8_t* data,
-    int length)
+static void print_hex_string(const uint8_t* data, int length)
 {
-  for (int i = 0; i < min(32, length); i++)
+  for (int i = 0; i < min(64, length); i++)
     printf("%s%02X", (i == 0 ? "" : " "), data[i]);
 
-  puts(length > 32 ? " ..." : "");
+  puts(length > 64 ? " ..." : "");
 }
 
-
-static void print_error(
-    int error)
+static void print_error(int error)
 {
   switch (error)
   {
-    case ERROR_SUCCESS:
-      break;
-    case ERROR_COULD_NOT_ATTACH_TO_PROCESS:
-      fprintf(stderr, "can not attach to process (try running as root)\n");
-      break;
-    case ERROR_INSUFFICIENT_MEMORY:
-      fprintf(stderr, "not enough memory\n");
-      break;
-    case ERROR_SCAN_TIMEOUT:
-      fprintf(stderr, "scanning timed out\n");
-      break;
-    case ERROR_COULD_NOT_OPEN_FILE:
-      fprintf(stderr, "could not open file\n");
-      break;
-    case ERROR_UNSUPPORTED_FILE_VERSION:
-      fprintf(stderr, "rules were compiled with a different version of YARA\n");
-      break;
-    case ERROR_INVALID_FILE:
-      fprintf(stderr, "invalid compiled rules file.\n");
-      break;
-    case ERROR_CORRUPT_FILE:
-      fprintf(stderr, "corrupt compiled rules file.\n");
-      break;
-    case ERROR_EXEC_STACK_OVERFLOW:
-      fprintf(stderr, "stack overflow while evaluating condition "
-                      "(see --stack-size argument) \n");
-      break;
-    case ERROR_INVALID_EXTERNAL_VARIABLE_TYPE:
-      fprintf(stderr, "invalid type for external variable\n");
-      break;
-    case ERROR_TOO_MANY_MATCHES:
-      fprintf(stderr, "too many matches\n");
-      break;
-    default:
-      fprintf(stderr, "error: %d\n", error);
-      break;
+  case ERROR_SUCCESS:
+    break;
+  case ERROR_COULD_NOT_ATTACH_TO_PROCESS:
+    fprintf(stderr, "can not attach to process (try running as root)\n");
+    break;
+  case ERROR_INSUFFICIENT_MEMORY:
+    fprintf(stderr, "not enough memory\n");
+    break;
+  case ERROR_SCAN_TIMEOUT:
+    fprintf(stderr, "scanning timed out\n");
+    break;
+  case ERROR_COULD_NOT_OPEN_FILE:
+    fprintf(stderr, "could not open file\n");
+    break;
+  case ERROR_UNSUPPORTED_FILE_VERSION:
+    fprintf(stderr, "rules were compiled with a different version of YARA\n");
+    break;
+  case ERROR_INVALID_FILE:
+    fprintf(stderr, "invalid compiled rules file.\n");
+    break;
+  case ERROR_CORRUPT_FILE:
+    fprintf(stderr, "corrupt compiled rules file.\n");
+    break;
+  case ERROR_EXEC_STACK_OVERFLOW:
+    fprintf(
+        stderr,
+        "stack overflow while evaluating condition "
+        "(see --stack-size argument) \n");
+    break;
+  case ERROR_INVALID_EXTERNAL_VARIABLE_TYPE:
+    fprintf(stderr, "invalid type for external variable\n");
+    break;
+  case ERROR_TOO_MANY_MATCHES:
+    fprintf(stderr, "too many matches\n");
+    break;
+  default:
+    fprintf(stderr, "error: %d\n", error);
+    break;
   }
 }
 
-static void print_scanner_error(
-    YR_SCANNER* scanner,
-    int error)
+static void print_scanner_error(YR_SCANNER* scanner, int error)
 {
   YR_RULE* rule = yr_scanner_last_error_rule(scanner);
   YR_STRING* string = yr_scanner_last_error_string(scanner);
@@ -695,16 +764,12 @@ static void print_scanner_error(
   }
   else if (rule != NULL)
   {
-    fprintf(
-        stderr,
-        "rule \"%s\" caused ",
-        rule->identifier);
+    fprintf(stderr, "rule \"%s\" caused ", rule->identifier);
   }
 
   print_error(error);
 }
 
-
 static void print_compiler_error(
     int error_level,
     const char* file_name,
@@ -734,28 +799,21 @@ static void print_compiler_error(
   {
     fprintf(
         stderr,
-        "%s(%d): %s in rule \"%s\": %s\n",
-        file_name,
-        line_number,
+        "%s: rule \"%s\" in %s(%d): %s\n",
         msg_type,
         rule->identifier,
+        file_name,
+        line_number,
         message);
   }
   else
   {
     fprintf(
-        stderr,
-        "%s(%d): %s: %s\n",
-        file_name,
-        line_number,
-        msg_type,
-        message);
+        stderr, "%s(%d): %s: %s\n", file_name, line_number, msg_type, message);
   }
 }
 
-
-static void print_rules_stats(
-    YR_RULES* rules)
+static void print_rules_stats(YR_RULES* rules)
 {
   YR_RULES_STATS stats;
 
@@ -766,29 +824,21 @@ static void print_rules_stats(
 
   if (result != ERROR_SUCCESS)
   {
-     print_error(result);
-     return;
+    print_error(result);
+    return;
   }
 
-  printf(
-      "size of AC transition table        : %d\n",
-      stats.ac_tables_size);
+  printf("size of AC transition table        : %d\n", stats.ac_tables_size);
 
   printf(
       "average length of AC matches lists : %f\n",
       stats.ac_average_match_list_length);
 
-  printf(
-      "number of rules                    : %d\n",
-      stats.num_rules);
+  printf("number of rules                    : %d\n", stats.num_rules);
 
-  printf(
-      "number of strings                  : %d\n",
-      stats.num_strings);
+  printf("number of strings                  : %d\n", stats.num_strings);
 
-  printf(
-      "number of AC matches               : %d\n",
-      stats.ac_matches);
+  printf("number of AC matches               : %d\n", stats.ac_matches);
 
   printf(
       "number of AC matches in root node  : %d\n",
@@ -805,7 +855,6 @@ static void print_rules_stats(
     printf(" %3d: %d\n", i, stats.ac_match_list_length_pctls[i]);
 }
 
-
 static int handle_message(
     YR_SCAN_CONTEXT* context,
     int message,
@@ -858,7 +907,7 @@ static int handle_message(
 
   if (show && !print_count_only)
   {
-    mutex_lock(&output_mutex);
+    cli_mutex_lock(&output_mutex);
 
     if (show_namespace)
       printf("%s:", rule->ns->name);
@@ -928,14 +977,16 @@ static int handle_message(
         yr_string_matches_foreach(context, string, match)
         {
           if (show_string_length)
-            printf("0x%" PRIx64 ":%d:%s",
-              match->base + match->offset,
-              match->data_length,
-              string->identifier);
+            printf(
+                "0x%" PRIx64 ":%d:%s",
+                match->base + match->offset,
+                match->data_length,
+                string->identifier);
           else
-            printf("0x%" PRIx64 ":%s",
-              match->base + match->offset,
-              string->identifier);
+            printf(
+                "0x%" PRIx64 ":%s",
+                match->base + match->offset,
+                string->identifier);
 
           if (show_strings)
           {
@@ -954,7 +1005,7 @@ static int handle_message(
       }
     }
 
-    mutex_unlock(&output_mutex);
+    cli_mutex_unlock(&output_mutex);
   }
 
   if (is_matching)
@@ -969,7 +1020,6 @@ static int handle_message(
   return CALLBACK_CONTINUE;
 }
 
-
 static int callback(
     YR_SCAN_CONTEXT* context,
     int message,
@@ -977,56 +1027,76 @@ static int callback(
     void* user_data)
 {
   YR_MODULE_IMPORT* mi;
+  YR_STRING* string;
+  YR_RULE* rule;
   YR_OBJECT* object;
   MODULE_DATA* module_data;
 
-  switch(message)
+  switch (message)
   {
-    case CALLBACK_MSG_RULE_MATCHING:
-    case CALLBACK_MSG_RULE_NOT_MATCHING:
-      return handle_message(
-          context, message, (YR_RULE*) message_data, user_data);
+  case CALLBACK_MSG_RULE_MATCHING:
+  case CALLBACK_MSG_RULE_NOT_MATCHING:
+    return handle_message(context, message, (YR_RULE*) message_data, user_data);
 
-    case CALLBACK_MSG_IMPORT_MODULE:
+  case CALLBACK_MSG_IMPORT_MODULE:
 
-      mi = (YR_MODULE_IMPORT*) message_data;
-      module_data = modules_data_list;
+    mi = (YR_MODULE_IMPORT*) message_data;
+    module_data = modules_data_list;
 
-      while (module_data != NULL)
+    while (module_data != NULL)
+    {
+      if (strcmp(module_data->module_name, mi->module_name) == 0)
       {
-        if (strcmp(module_data->module_name, mi->module_name) == 0)
-        {
-          mi->module_data = (void*) module_data->mapped_file.data;
-          mi->module_data_size = module_data->mapped_file.size;
-          break;
-        }
-
-        module_data = module_data->next;
+        mi->module_data = (void*) module_data->mapped_file.data;
+        mi->module_data_size = module_data->mapped_file.size;
+        break;
       }
 
+      module_data = module_data->next;
+    }
+
+    return CALLBACK_CONTINUE;
+
+  case CALLBACK_MSG_MODULE_IMPORTED:
+
+    if (show_module_data)
+    {
+      object = (YR_OBJECT*) message_data;
+
+      cli_mutex_lock(&output_mutex);
+
+      yr_object_print_data(object, 0, 1);
+      printf("\n");
+
+      cli_mutex_unlock(&output_mutex);
+    }
+
+    return CALLBACK_CONTINUE;
+
+  case CALLBACK_MSG_TOO_MANY_MATCHES:
+
+    if (ignore_warnings)
       return CALLBACK_CONTINUE;
 
-    case CALLBACK_MSG_MODULE_IMPORTED:
+    string = (YR_STRING*) message_data;
+    rule = &context->rules->rules_table[string->rule_idx];
 
-      if (show_module_data)
-      {
-        object = (YR_OBJECT*) message_data;
+    fprintf(
+        stderr,
+        "warning: rule \"%s\": too many matches for %s, results for this rule "
+        "may be incorrect\n",
+        rule->identifier,
+        string->identifier);
 
-        mutex_lock(&output_mutex);
+    if (fail_on_warnings)
+      return CALLBACK_ERROR;
 
-        yr_object_print_data(object, 0, 1);
-        printf("\n");
-
-        mutex_unlock(&output_mutex);
-      }
-
-      return CALLBACK_CONTINUE;
+    return CALLBACK_CONTINUE;
   }
 
   return CALLBACK_ERROR;
 }
 
-
 #if defined(_WIN32) || defined(__CYGWIN__)
 static DWORD WINAPI scanning_thread(LPVOID param)
 #else
@@ -1035,38 +1105,39 @@ static void* scanning_thread(void* param)
 {
   int result = ERROR_SUCCESS;
   THREAD_ARGS* args = (THREAD_ARGS*) param;
-  char* file_path = file_queue_get();
+
+  char* file_path = file_queue_get(args->deadline);
 
   while (file_path != NULL)
   {
     args->callback_args.current_count = 0;
     args->callback_args.file_path = file_path;
 
-    int elapsed_time = (int) difftime(time(NULL), args->start_time);
+    time_t current_time = time(NULL);
 
-    if (elapsed_time < timeout)
+    if (current_time < args->deadline)
     {
-      yr_scanner_set_timeout(args->scanner, timeout - elapsed_time);
+      yr_scanner_set_timeout(args->scanner, args->deadline - current_time);
 
       result = yr_scanner_scan_file(args->scanner, file_path);
 
       if (print_count_only)
       {
-        mutex_lock(&output_mutex);
+        cli_mutex_lock(&output_mutex);
         printf("%s: %d\n", file_path, args->callback_args.current_count);
-        mutex_unlock(&output_mutex);
+        cli_mutex_unlock(&output_mutex);
       }
 
       if (result != ERROR_SUCCESS)
       {
-        mutex_lock(&output_mutex);
+        cli_mutex_lock(&output_mutex);
         fprintf(stderr, "error scanning %s: ", file_path);
         print_scanner_error(args->scanner, result);
-        mutex_unlock(&output_mutex);
+        cli_mutex_unlock(&output_mutex);
       }
 
       free(file_path);
-      file_path = file_queue_get();
+      file_path = file_queue_get(args->deadline);
     }
     else
     {
@@ -1077,10 +1148,7 @@ static void* scanning_thread(void* param)
   return 0;
 }
 
-
-static int define_external_variables(
-    YR_RULES* rules,
-    YR_COMPILER* compiler)
+static int define_external_variables(YR_RULES* rules, YR_COMPILER* compiler)
 {
   int result = ERROR_SUCCESS;
 
@@ -1106,65 +1174,46 @@ static int define_external_variables(
     if (is_float(value))
     {
       if (rules != NULL)
-        result = yr_rules_define_float_variable(
-            rules,
-            identifier,
-            atof(value));
+        result = yr_rules_define_float_variable(rules, identifier, atof(value));
 
       if (compiler != NULL)
         result = yr_compiler_define_float_variable(
-            compiler,
-            identifier,
-            atof(value));
+            compiler, identifier, atof(value));
     }
     else if (is_integer(value))
     {
       if (rules != NULL)
         result = yr_rules_define_integer_variable(
-            rules,
-            identifier,
-            atoi(value));
+            rules, identifier, atoi(value));
 
       if (compiler != NULL)
         result = yr_compiler_define_integer_variable(
-            compiler,
-            identifier,
-            atoi(value));
+            compiler, identifier, atoi(value));
     }
     else if (strcmp(value, "true") == 0 || strcmp(value, "false") == 0)
     {
       if (rules != NULL)
         result = yr_rules_define_boolean_variable(
-            rules,
-            identifier,
-            strcmp(value, "true") == 0);
+            rules, identifier, strcmp(value, "true") == 0);
 
       if (compiler != NULL)
         result = yr_compiler_define_boolean_variable(
-            compiler,
-            identifier,
-            strcmp(value, "true") == 0);
+            compiler, identifier, strcmp(value, "true") == 0);
     }
     else
     {
       if (rules != NULL)
-        result = yr_rules_define_string_variable(
-            rules,
-            identifier,
-            value);
+        result = yr_rules_define_string_variable(rules, identifier, value);
 
       if (compiler != NULL)
         result = yr_compiler_define_string_variable(
-            compiler,
-            identifier,
-            value);
+            compiler, identifier, value);
     }
   }
 
   return result;
 }
 
-
 static int load_modules_data()
 {
   for (int i = 0; modules_data[i] != NULL; i++)
@@ -1202,12 +1251,11 @@ static int load_modules_data()
   return true;
 }
 
-
 static void unload_modules_data()
 {
   MODULE_DATA* module_data = modules_data_list;
 
-  while(module_data != NULL)
+  while (module_data != NULL)
   {
     MODULE_DATA* next_module_data = module_data->next;
 
@@ -1220,23 +1268,24 @@ static void unload_modules_data()
   modules_data_list = NULL;
 }
 
-
-int main(
-    int argc,
-    const char** argv)
+int main(int argc, const char** argv)
 {
   COMPILER_RESULTS cr;
 
   YR_COMPILER* compiler = NULL;
   YR_RULES* rules = NULL;
   YR_SCANNER* scanner = NULL;
+  SCAN_OPTIONS scan_opts;
 
   bool arg_is_dir = false;
   int flags = 0;
-  int result, i;
+  int result;
 
   argc = args_parse(options, argc, argv);
 
+  scan_opts.follow_symlinks = follow_symlinks;
+  scan_opts.recursive_search = recursive_search;
+
   if (show_version)
   {
     printf("%s\n", YR_VERSION);
@@ -1246,13 +1295,16 @@ int main(
   if (show_help)
   {
     printf(
-      "YARA %s, the pattern matching swiss army knife.\n"
-      "%s\n\n"
-      "Mandatory arguments to long options are mandatory for "
-      "short options too.\n\n", YR_VERSION, USAGE_STRING);
+        "YARA %s, the pattern matching swiss army knife.\n"
+        "%s\n\n"
+        "Mandatory arguments to long options are mandatory for "
+        "short options too.\n\n",
+        YR_VERSION,
+        USAGE_STRING);
 
     args_print_usage(options, 40);
-    printf("\nSend bug reports and suggestions to: vmalvarez@virustotal.com.\n");
+    printf(
+        "\nSend bug reports and suggestions to: vmalvarez@virustotal.com.\n");
 
     return EXIT_SUCCESS;
   }
@@ -1300,9 +1352,10 @@ int main(
 
     if (argc != 2)
     {
-      fprintf(stderr,
-        "error: can't accept multiple rules files if one of them is in "
-        "compiled form.\n");
+      fprintf(
+          stderr,
+          "error: can't accept multiple rules files if one of them is in "
+          "compiled form.\n");
       exit_with_code(EXIT_FAILURE);
     }
 
@@ -1370,11 +1423,13 @@ int main(
   if (show_stats)
     print_rules_stats(rules);
 
-  mutex_init(&output_mutex);
+  cli_mutex_init(&output_mutex);
 
   if (fast_scan)
     flags |= SCAN_FLAGS_FAST_MODE;
 
+  scan_opts.deadline = time(NULL) + timeout;
+
   arg_is_dir = is_directory(argv[argc - 1]);
 
   if (scan_list_search && arg_is_dir)
@@ -1393,11 +1448,9 @@ int main(
     THREAD thread[YR_MAX_THREADS];
     THREAD_ARGS thread_args[YR_MAX_THREADS];
 
-    time_t start_time = time(NULL);
-
-    for (i = 0; i < threads; i++)
+    for (int i = 0; i < threads; i++)
     {
-      thread_args[i].start_time = start_time;
+      thread_args[i].deadline = scan_opts.deadline;
       thread_args[i].current_count = 0;
 
       result = yr_scanner_create(rules, &thread_args[i].scanner);
@@ -1409,13 +1462,12 @@ int main(
       }
 
       yr_scanner_set_callback(
-          thread_args[i].scanner,
-          callback,
-          &thread_args[i].callback_args);
+          thread_args[i].scanner, callback, &thread_args[i].callback_args);
 
       yr_scanner_set_flags(thread_args[i].scanner, flags);
 
-      if (create_thread(&thread[i], scanning_thread, (void*) &thread_args[i]))
+      if (cli_create_thread(
+              &thread[i], scanning_thread, (void*) &thread_args[i]))
       {
         print_error(ERROR_COULD_NOT_CREATE_THREAD);
         exit_with_code(EXIT_FAILURE);
@@ -1424,11 +1476,11 @@ int main(
 
     if (arg_is_dir)
     {
-      scan_dir(argv[argc - 1], recursive_search, start_time);
+      scan_dir(argv[argc - 1], &scan_opts);
     }
     else
     {
-      result = populate_scan_list(argv[argc - 1], recursive_search, start_time);
+      result = populate_scan_list(argv[argc - 1], &scan_opts);
 
       if (result != ERROR_SUCCESS)
         exit_with_code(EXIT_FAILURE);
@@ -1437,17 +1489,16 @@ int main(
     file_queue_finish();
 
     // Wait for scan threads to finish
-    for (i = 0; i < threads; i++)
-      thread_join(&thread[i]);
+    for (int i = 0; i < threads; i++) cli_thread_join(&thread[i]);
 
-    for (i = 0; i < threads; i++)
+    for (int i = 0; i < threads; i++)
       yr_scanner_destroy(thread_args[i].scanner);
 
     file_queue_destroy();
   }
   else
   {
-    CALLBACK_ARGS user_data = { argv[argc - 1], 0 };
+    CALLBACK_ARGS user_data = {argv[argc - 1], 0};
 
     result = yr_scanner_create(rules, &scanner);
 
@@ -1476,9 +1527,9 @@ int main(
     if (print_count_only)
       printf("%d\n", user_data.current_count);
 
-    #ifdef YR_PROFILING_ENABLED
+#ifdef YR_PROFILING_ENABLED
     yr_scanner_print_profiling_info(scanner);
-    #endif
+#endif
   }
 
   result = EXIT_SUCCESS;
