@@ -41,16 +41,15 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <unistd.h>
 #endif
 
-#include <yara/libyara.h>
-#include <yara/utils.h>
 #include <yara/compiler.h>
-#include <yara/exec.h>
 #include <yara/error.h>
+#include <yara/exec.h>
+#include <yara/lexer.h>
+#include <yara/libyara.h>
 #include <yara/mem.h>
 #include <yara/object.h>
-#include <yara/lexer.h>
 #include <yara/strutils.h>
-
+#include <yara/utils.h>
 
 static void _yr_compiler_default_include_free(
     const char* callback_result_ptr,
@@ -58,67 +57,66 @@ static void _yr_compiler_default_include_free(
 {
   if (callback_result_ptr != NULL)
   {
-    yr_free((void*)callback_result_ptr);
+    yr_free((void*) callback_result_ptr);
   }
 }
 
-
 const char* _yr_compiler_default_include_callback(
     const char* include_name,
     const char* calling_rule_filename,
     const char* calling_rule_namespace,
     void* user_data)
 {
-  #ifndef _MSC_VER
+#ifndef _MSC_VER
   struct stat stbuf;
-  #endif
+#endif
 
   char* file_buffer;
 
-  #ifdef _MSC_VER
+#ifdef _MSC_VER
   long file_size;
-  #else
+#else
   off_t file_size;
-  #endif
+#endif
 
   int fd = -1;
 
-  #if defined(_MSC_VER)
+#if defined(_MSC_VER)
   _sopen_s(&fd, include_name, _O_RDONLY | _O_BINARY, _SH_DENYWR, _S_IREAD);
-  #elif defined(_WIN32) || defined(__CYGWIN__)
+#elif defined(_WIN32) || defined(__CYGWIN__)
   fd = open(include_name, O_RDONLY | O_BINARY);
-  #else
+#else
   fd = open(include_name, O_RDONLY);
-  #endif
+#endif
 
   if (fd == -1)
     return NULL;
 
-  #ifdef _MSC_VER
+#ifdef _MSC_VER
   file_size = _filelength(fd);
   if (file_size == -1)
   {
     _close(fd);
     return NULL;
   }
-  #else
+#else
   if ((fstat(fd, &stbuf) != 0) || (!S_ISREG(stbuf.st_mode)))
   {
     close(fd);
     return NULL;
   }
   file_size = stbuf.st_size;
-  #endif
+#endif
 
   file_buffer = (char*) yr_malloc((size_t) file_size + 1);
 
   if (file_buffer == NULL)
   {
-    #ifdef _MSC_VER
+#ifdef _MSC_VER
     _close(fd);
-    #else
+#else
     close(fd);
-    #endif
+#endif
 
     return NULL;
   }
@@ -127,11 +125,11 @@ const char* _yr_compiler_default_include_callback(
   {
     yr_free(file_buffer);
 
-    #ifdef _MSC_VER
+#ifdef _MSC_VER
     _close(fd);
-    #else
+#else
     close(fd);
-    #endif
+#endif
 
     return NULL;
   }
@@ -140,34 +138,83 @@ const char* _yr_compiler_default_include_callback(
     file_buffer[file_size] = '\0';
   }
 
-  #ifdef _MSC_VER
+#ifdef _MSC_VER
   _close(fd);
-  #else
+#else
   close(fd);
-  #endif
+#endif
 
   return file_buffer;
 }
 
-
+////////////////////////////////////////////////////////////////////////////////
+// Returns a rule given its index in the rules table.
 //
-// _yr_compiler_get_rule_by_idx returns a rule given its index in the rules
-// table. The returned pointer is valid as long as no other rule is written
-// to the table. This is because the write operation may cause the table to
-// be moved to a different location in memory. Use the pointer only in a
-// limited scope where you can be sure that no other rule is being written
-// during the pointer's lifetime.
+// The returned pointer is valid as long as no other rule is written to the
+// table. This is because the write operation may cause the table to be moved to
+// a different location in memory. Use the pointer only in a limited scope where
+// you can be sure that no other rule is being written during the pointer's
+// lifetime.
 //
-YR_RULE* _yr_compiler_get_rule_by_idx(
-    YR_COMPILER* compiler, uint32_t rule_idx)
+YR_RULE* _yr_compiler_get_rule_by_idx(YR_COMPILER* compiler, uint32_t rule_idx)
 {
   return (YR_RULE*) yr_arena_get_ptr(
       compiler->arena, YR_RULES_TABLE, rule_idx * sizeof(YR_RULE));
 }
 
+////////////////////////////////////////////////////////////////////////////////
+// Stores some data in the YR_SZ_POOL and returns a reference to it.
+//
+// If the same data was already stored in a previous call to this function the
+// data is not written again, a reference to the existing data is returned
+// instead.
+//
+int _yr_compiler_store_data(
+    YR_COMPILER* compiler,
+    const void* data,
+    size_t data_length,
+    YR_ARENA_REF* ref)
+{
+  // Check if the data is already in YR_SZ_POOL by using a hash table.
+  uint32_t offset = yr_hash_table_lookup_uint32_raw_key(
+      compiler->sz_table, data, data_length, NULL);
 
-YR_API int yr_compiler_create(
-    YR_COMPILER** compiler)
+  if (offset == UINT32_MAX)
+  {
+    // The data was not previously written to YR_SZ_POOL, write it and store
+    // the reference's offset in the hash table. Storing the buffer number
+    // is not necessary, it's always YR_SZ_POOL.
+    FAIL_ON_ERROR(yr_arena_write_data(
+        compiler->arena, YR_SZ_POOL, data, data_length, ref));
+
+    FAIL_ON_ERROR(yr_hash_table_add_uint32_raw_key(
+        compiler->sz_table, data, data_length, NULL, ref->offset));
+  }
+  else
+  {
+    ref->buffer_id = YR_SZ_POOL;
+    ref->offset = offset;
+  }
+
+  return ERROR_SUCCESS;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// Similar to _yr_compiler_store_data, but receives a null-terminated string.
+//
+int _yr_compiler_store_string(
+    YR_COMPILER* compiler,
+    const char* string,
+    YR_ARENA_REF* ref)
+{
+  return _yr_compiler_store_data(
+      compiler,
+      (void*) string,
+      strlen(string) + 1,  // include the null terminator
+      ref);
+}
+
+YR_API int yr_compiler_create(YR_COMPILER** compiler)
 {
   int result;
   YR_COMPILER* new_compiler;
@@ -200,25 +247,26 @@ YR_API int yr_compiler_create(
   new_compiler->loop_for_of_var_index = -1;
 
   new_compiler->atoms_config.get_atom_quality = yr_atoms_heuristic_quality;
-  new_compiler->atoms_config.quality_warning_threshold = \
+  new_compiler->atoms_config.quality_warning_threshold =
       YR_ATOM_QUALITY_WARNING_THRESHOLD;
 
-  result = yr_hash_table_create(10007, &new_compiler->rules_table);
+  result = yr_hash_table_create(5000, &new_compiler->rules_table);
 
   if (result == ERROR_SUCCESS)
-    result = yr_hash_table_create(10007, &new_compiler->objects_table);
+    result = yr_hash_table_create(1000, &new_compiler->objects_table);
 
   if (result == ERROR_SUCCESS)
-    result = yr_hash_table_create(101, &new_compiler->strings_table);
+    result = yr_hash_table_create(10000, &new_compiler->strings_table);
 
   if (result == ERROR_SUCCESS)
-    result = yr_arena_create(
-        YR_NUM_SECTIONS, 1048576, &new_compiler->arena);
+    result = yr_hash_table_create(10000, &new_compiler->sz_table);
+
+  if (result == ERROR_SUCCESS)
+    result = yr_arena_create(YR_NUM_SECTIONS, 1048576, &new_compiler->arena);
 
   if (result == ERROR_SUCCESS)
     result = yr_ac_automaton_create(
-        new_compiler->arena,
-        &new_compiler->automaton);
+        new_compiler->arena, &new_compiler->automaton);
 
   if (result == ERROR_SUCCESS)
   {
@@ -232,28 +280,24 @@ YR_API int yr_compiler_create(
   return result;
 }
 
-
-YR_API void yr_compiler_destroy(
-    YR_COMPILER* compiler)
+YR_API void yr_compiler_destroy(YR_COMPILER* compiler)
 {
   yr_arena_release(compiler->arena);
 
   if (compiler->automaton != NULL)
     yr_ac_automaton_destroy(compiler->automaton);
 
-  yr_hash_table_destroy(
-      compiler->rules_table,
-      NULL);
+  yr_hash_table_destroy(compiler->rules_table, NULL);
 
-  yr_hash_table_destroy(
-      compiler->strings_table,
-      NULL);
+  yr_hash_table_destroy(compiler->strings_table, NULL);
+
+  yr_hash_table_destroy(compiler->sz_table, NULL);
 
   yr_hash_table_destroy(
       compiler->objects_table,
       (YR_HASH_TABLE_FREE_VALUE_FUNC) yr_object_destroy);
 
-  if (compiler->  atoms_config.free_quality_table)
+  if (compiler->atoms_config.free_quality_table)
     yr_free(compiler->atoms_config.quality_table);
 
   for (int i = 0; i < compiler->file_name_stack_ptr; i++)
@@ -271,7 +315,6 @@ YR_API void yr_compiler_destroy(
   yr_free(compiler);
 }
 
-
 YR_API void yr_compiler_set_callback(
     YR_COMPILER* compiler,
     YR_COMPILER_CALLBACK_FUNC callback,
@@ -281,7 +324,6 @@ YR_API void yr_compiler_set_callback(
   compiler->user_data = user_data;
 }
 
-
 YR_API void yr_compiler_set_include_callback(
     YR_COMPILER* compiler,
     YR_COMPILER_INCLUDE_CALLBACK_FUNC include_callback,
@@ -293,7 +335,6 @@ YR_API void yr_compiler_set_include_callback(
   compiler->incl_clbk_user_data = user_data;
 }
 
-
 YR_API void yr_compiler_set_re_ast_callback(
     YR_COMPILER* compiler,
     YR_COMPILER_RE_AST_CALLBACK_FUNC re_ast_callback,
@@ -303,10 +344,7 @@ YR_API void yr_compiler_set_re_ast_callback(
   compiler->re_ast_clbk_user_data = user_data;
 }
 
-
-//
-// yr_compiler_set_atom_quality_table
-//
+////////////////////////////////////////////////////////////////////////////////
 // This function allows to specify an atom quality table to be used by the
 // compiler for choosing the best atoms from regular expressions and strings.
 // When a quality table is set, the compiler uses yr_atoms_table_quality
@@ -333,7 +371,7 @@ YR_API void yr_compiler_set_re_ast_callback(
 // The "warning_threshold" argument must be a number between 0 and 255, if some
 // atom chosen for a string have a quality below the specified threshold a
 // warning like "<string> is slowing down scanning" is shown.
-
+//
 YR_API void yr_compiler_set_atom_quality_table(
     YR_COMPILER* compiler,
     const void* table,
@@ -344,17 +382,13 @@ YR_API void yr_compiler_set_atom_quality_table(
   compiler->atoms_config.quality_warning_threshold = warning_threshold;
   compiler->atoms_config.get_atom_quality = yr_atoms_table_quality;
   compiler->atoms_config.quality_table_entries = entries;
-  compiler->atoms_config.quality_table = \
-      (YR_ATOM_QUALITY_TABLE_ENTRY*) table;
+  compiler->atoms_config.quality_table = (YR_ATOM_QUALITY_TABLE_ENTRY*) table;
 }
 
-//
-// yr_compiler_set_atom_quality_table
-//
+////////////////////////////////////////////////////////////////////////////////
 // Load an atom quality table from a file. The file's content must have the
 // format explained in the description for yr_compiler_set_atom_quality_table.
 //
-
 YR_API int yr_compiler_load_atom_quality_table(
     YR_COMPILER* compiler,
     const char* filename,
@@ -406,10 +440,7 @@ YR_API int yr_compiler_load_atom_quality_table(
   return ERROR_SUCCESS;
 }
 
-
-int _yr_compiler_push_file_name(
-    YR_COMPILER* compiler,
-    const char* file_name)
+int _yr_compiler_push_file_name(YR_COMPILER* compiler, const char* file_name)
 {
   char* str;
   int i;
@@ -434,9 +465,7 @@ int _yr_compiler_push_file_name(
   return ERROR_SUCCESS;
 }
 
-
-void _yr_compiler_pop_file_name(
-    YR_COMPILER* compiler)
+void _yr_compiler_pop_file_name(YR_COMPILER* compiler)
 {
   if (compiler->file_name_stack_ptr > 0)
   {
@@ -446,9 +475,7 @@ void _yr_compiler_pop_file_name(
   }
 }
 
-
-int _yr_compiler_get_var_frame(
-    YR_COMPILER* compiler)
+int _yr_compiler_get_var_frame(YR_COMPILER* compiler)
 {
   int i, result = 0;
 
@@ -461,9 +488,7 @@ int _yr_compiler_get_var_frame(
   return result;
 }
 
-
-YR_API char* yr_compiler_get_current_file_name(
-    YR_COMPILER* compiler)
+YR_API char* yr_compiler_get_current_file_name(YR_COMPILER* compiler)
 {
   if (compiler->file_name_stack_ptr > 0)
   {
@@ -475,7 +500,6 @@ YR_API char* yr_compiler_get_current_file_name(
   }
 }
 
-
 static int _yr_compiler_set_namespace(
     YR_COMPILER* compiler,
     const char* namespace_)
@@ -509,11 +533,7 @@ static int _yr_compiler_set_namespace(
 
     ns = (YR_NAMESPACE*) yr_arena_ref_to_ptr(compiler->arena, &ref);
 
-    FAIL_ON_ERROR(yr_arena_write_string(
-        compiler->arena,
-        YR_SZ_POOL,
-        namespace_,
-        &ref));
+    FAIL_ON_ERROR(_yr_compiler_store_string(compiler, namespace_, &ref));
 
     ns->name = (const char*) yr_arena_ref_to_ptr(compiler->arena, &ref);
     ns->idx = compiler->num_namespaces;
@@ -525,7 +545,6 @@ static int _yr_compiler_set_namespace(
   return ERROR_SUCCESS;
 }
 
-
 YR_API int yr_compiler_add_file(
     YR_COMPILER* compiler,
     FILE* rules_file,
@@ -563,7 +582,6 @@ YR_API int yr_compiler_add_file(
   return result;
 }
 
-
 YR_API int yr_compiler_add_fd(
     YR_COMPILER* compiler,
     YR_FILE_DESCRIPTOR rules_fd,
@@ -574,12 +592,10 @@ YR_API int yr_compiler_add_fd(
 
   // Don't allow yr_compiler_add_fd() after
   // yr_compiler_get_rules() has been called.
-
   assert(compiler->rules == NULL);
 
   // Don't allow calls to yr_compiler_add_fd() if a previous call to
   // yr_compiler_add_XXXX failed.
-
   assert(compiler->errors == 0);
 
   if (namespace_ != NULL)
@@ -601,7 +617,6 @@ YR_API int yr_compiler_add_fd(
   return result;
 }
 
-
 YR_API int yr_compiler_add_string(
     YR_COMPILER* compiler,
     const char* rules_string,
@@ -609,12 +624,10 @@ YR_API int yr_compiler_add_string(
 {
   // Don't allow calls to yr_compiler_add_string() after
   // yr_compiler_get_rules() has been called.
-
   assert(compiler->rules == NULL);
 
   // Don't allow calls to yr_compiler_add_string() if a previous call to
   // yr_compiler_add_XXXX failed.
-
   assert(compiler->errors == 0);
 
   if (namespace_ != NULL)
@@ -628,9 +641,7 @@ YR_API int yr_compiler_add_string(
   return yr_lex_parse_rules_string(rules_string, compiler);
 }
 
-
-static int _yr_compiler_compile_rules(
-    YR_COMPILER* compiler)
+static int _yr_compiler_compile_rules(YR_COMPILER* compiler)
 {
   YR_RULE null_rule;
   YR_EXTERNAL_VARIABLE null_external;
@@ -639,22 +650,14 @@ static int _yr_compiler_compile_rules(
 
   // Write halt instruction at the end of code.
   FAIL_ON_ERROR(yr_arena_write_data(
-      compiler->arena,
-      YR_CODE_SECTION,
-      &halt,
-      sizeof(uint8_t),
-      NULL));
+      compiler->arena, YR_CODE_SECTION, &halt, sizeof(uint8_t), NULL));
 
   // Write a null rule indicating the end.
   memset(&null_rule, 0xFA, sizeof(YR_RULE));
   null_rule.flags = RULE_FLAGS_NULL;
 
   FAIL_ON_ERROR(yr_arena_write_data(
-    compiler->arena,
-    YR_RULES_TABLE,
-    &null_rule,
-    sizeof(YR_RULE),
-    NULL));
+      compiler->arena, YR_RULES_TABLE, &null_rule, sizeof(YR_RULE), NULL));
 
   // Write a null external indicating the end.
   memset(&null_external, 0xFA, sizeof(YR_EXTERNAL_VARIABLE));
@@ -668,18 +671,12 @@ static int _yr_compiler_compile_rules(
       NULL));
 
   // Write Aho-Corasick automaton to arena.
-  FAIL_ON_ERROR(yr_ac_compile(
-      compiler->automaton,
-      compiler->arena));
+  FAIL_ON_ERROR(yr_ac_compile(compiler->automaton, compiler->arena));
 
   YR_ARENA_REF ref;
 
   FAIL_ON_ERROR(yr_arena_allocate_struct(
-      compiler->arena,
-      YR_SUMMARY_SECTION,
-      sizeof(YR_SUMMARY),
-      &ref,
-      EOL));
+      compiler->arena, YR_SUMMARY_SECTION, sizeof(YR_SUMMARY), &ref, EOL));
 
   YR_SUMMARY* summary = (YR_SUMMARY*) yr_arena_ref_to_ptr(
       compiler->arena, &ref);
@@ -691,10 +688,7 @@ static int _yr_compiler_compile_rules(
   return yr_rules_from_arena(compiler->arena, &compiler->rules);
 }
 
-
-YR_API int yr_compiler_get_rules(
-    YR_COMPILER* compiler,
-    YR_RULES** rules)
+YR_API int yr_compiler_get_rules(YR_COMPILER* compiler, YR_RULES** rules)
 {
   // Don't allow calls to yr_compiler_get_rules() if a previous call to
   // yr_compiler_add_XXXX failed.
@@ -703,7 +697,7 @@ YR_API int yr_compiler_get_rules(
   *rules = NULL;
 
   if (compiler->rules == NULL)
-     FAIL_ON_ERROR(_yr_compiler_compile_rules(compiler));
+    FAIL_ON_ERROR(_yr_compiler_compile_rules(compiler));
 
   *rules = compiler->rules;
 
@@ -721,9 +715,7 @@ static int _yr_compiler_define_variable(
     return ERROR_INVALID_ARGUMENT;
 
   object = (YR_OBJECT*) yr_hash_table_lookup(
-      compiler->objects_table,
-      external->identifier,
-      NULL);
+      compiler->objects_table, external->identifier, NULL);
 
   if (object != NULL)
     return ERROR_DUPLICATED_EXTERNAL_VARIABLE;
@@ -739,17 +731,12 @@ static int _yr_compiler_define_variable(
       offsetof(YR_EXTERNAL_VARIABLE, identifier),
       EOL));
 
-  ext = (YR_EXTERNAL_VARIABLE*) yr_arena_ref_to_ptr(
-      compiler->arena, &ext_ref);
+  ext = (YR_EXTERNAL_VARIABLE*) yr_arena_ref_to_ptr(compiler->arena, &ext_ref);
 
-  FAIL_ON_ERROR(yr_arena_write_string(
-      compiler->arena,
-      YR_SZ_POOL,
-      external->identifier,
-      &ref));
+  FAIL_ON_ERROR(
+      _yr_compiler_store_string(compiler, external->identifier, &ref));
 
-  ext->identifier = (const char*) yr_arena_ref_to_ptr(
-      compiler->arena, &ref);
+  ext->identifier = (const char*) yr_arena_ref_to_ptr(compiler->arena, &ref);
 
   ext->type = external->type;
   ext->value = external->value;
@@ -759,11 +746,7 @@ static int _yr_compiler_define_variable(
     if (external->value.s == NULL)
       return ERROR_INVALID_ARGUMENT;
 
-    FAIL_ON_ERROR(yr_arena_write_string(
-        compiler->arena,
-        YR_SZ_POOL,
-        external->value.s,
-        &ref));
+    FAIL_ON_ERROR(_yr_compiler_store_string(compiler, external->value.s, &ref));
 
     FAIL_ON_ERROR(yr_arena_make_ptr_relocatable(
         compiler->arena,
@@ -771,26 +754,19 @@ static int _yr_compiler_define_variable(
         ext_ref.offset + offsetof(YR_EXTERNAL_VARIABLE, value.s),
         EOL));
 
-    ext->value.s = (char*) yr_arena_ref_to_ptr(
-        compiler->arena, &ref);
+    ext->value.s = (char*) yr_arena_ref_to_ptr(compiler->arena, &ref);
   }
 
-  FAIL_ON_ERROR(yr_object_from_external_variable(
-      external,
-      &object));
+  FAIL_ON_ERROR(yr_object_from_external_variable(external, &object));
 
-  FAIL_ON_ERROR_WITH_CLEANUP(yr_hash_table_add(
-      compiler->objects_table,
-      external->identifier,
-      NULL,
-      (void*) object),
-      // cleanup
+  FAIL_ON_ERROR_WITH_CLEANUP(
+      yr_hash_table_add(
+          compiler->objects_table, external->identifier, NULL, (void*) object),
       yr_object_destroy(object));
 
   return ERROR_SUCCESS;
 }
 
-
 YR_API int yr_compiler_define_integer_variable(
     YR_COMPILER* compiler,
     const char* identifier,
@@ -802,13 +778,11 @@ YR_API int yr_compiler_define_integer_variable(
   external.identifier = identifier;
   external.value.i = value;
 
-  FAIL_ON_ERROR(_yr_compiler_define_variable(
-      compiler, &external));
+  FAIL_ON_ERROR(_yr_compiler_define_variable(compiler, &external));
 
   return ERROR_SUCCESS;
 }
 
-
 YR_API int yr_compiler_define_boolean_variable(
     YR_COMPILER* compiler,
     const char* identifier,
@@ -820,13 +794,11 @@ YR_API int yr_compiler_define_boolean_variable(
   external.identifier = identifier;
   external.value.i = value;
 
-  FAIL_ON_ERROR(_yr_compiler_define_variable(
-      compiler, &external));
+  FAIL_ON_ERROR(_yr_compiler_define_variable(compiler, &external));
 
   return ERROR_SUCCESS;
 }
 
-
 YR_API int yr_compiler_define_float_variable(
     YR_COMPILER* compiler,
     const char* identifier,
@@ -838,13 +810,11 @@ YR_API int yr_compiler_define_float_variable(
   external.identifier = identifier;
   external.value.f = value;
 
-  FAIL_ON_ERROR(_yr_compiler_define_variable(
-      compiler, &external));
+  FAIL_ON_ERROR(_yr_compiler_define_variable(compiler, &external));
 
   return ERROR_SUCCESS;
 }
 
-
 YR_API int yr_compiler_define_string_variable(
     YR_COMPILER* compiler,
     const char* identifier,
@@ -856,13 +826,11 @@ YR_API int yr_compiler_define_string_variable(
   external.identifier = identifier;
   external.value.s = (char*) value;
 
-  FAIL_ON_ERROR(_yr_compiler_define_variable(
-      compiler, &external));
+  FAIL_ON_ERROR(_yr_compiler_define_variable(compiler, &external));
 
   return ERROR_SUCCESS;
 }
 
-
 YR_API char* yr_compiler_get_error_message(
     YR_COMPILER* compiler,
     char* buffer,
@@ -870,231 +838,190 @@ YR_API char* yr_compiler_get_error_message(
 {
   uint32_t max_strings_per_rule;
 
-  switch(compiler->last_error)
+  switch (compiler->last_error)
   {
-    case ERROR_INSUFFICIENT_MEMORY:
-      snprintf(buffer, buffer_size, "not enough memory");
-      break;
-    case ERROR_DUPLICATED_IDENTIFIER:
-      snprintf(
-          buffer,
-          buffer_size,
-          "duplicated identifier \"%s\"",
-          compiler->last_error_extra_info);
-      break;
-    case ERROR_DUPLICATED_STRING_IDENTIFIER:
-      snprintf(
-          buffer,
-          buffer_size,
-          "duplicated string identifier \"%s\"",
-          compiler->last_error_extra_info);
-      break;
-    case ERROR_DUPLICATED_TAG_IDENTIFIER:
-      snprintf(
-          buffer,
-          buffer_size,
-          "duplicated tag identifier \"%s\"",
-          compiler->last_error_extra_info);
-      break;
-    case ERROR_DUPLICATED_META_IDENTIFIER:
-      snprintf(
-          buffer,
-          buffer_size,
-          "duplicated metadata identifier \"%s\"",
-          compiler->last_error_extra_info);
-      break;
-    case ERROR_DUPLICATED_LOOP_IDENTIFIER:
-      snprintf(
-          buffer,
-          buffer_size,
-          "duplicated loop identifier \"%s\"",
-          compiler->last_error_extra_info);
-      break;
-    case ERROR_UNDEFINED_STRING:
-      snprintf(
-          buffer,
-          buffer_size,
-          "undefined string \"%s\"",
-          compiler->last_error_extra_info);
-      break;
-    case ERROR_UNDEFINED_IDENTIFIER:
-      snprintf(
-          buffer,
-          buffer_size,
-          "undefined identifier \"%s\"",
-          compiler->last_error_extra_info);
-      break;
-    case ERROR_UNREFERENCED_STRING:
-      snprintf(
-          buffer,
-          buffer_size,
-          "unreferenced string \"%s\"",
-          compiler->last_error_extra_info);
-      break;
-    case ERROR_EMPTY_STRING:
-      snprintf(
-          buffer,
-          buffer_size,
-          "empty string \"%s\"",
-          compiler->last_error_extra_info);
-      break;
-    case ERROR_NOT_A_STRUCTURE:
-      snprintf(
-          buffer,
-          buffer_size,
-          "\"%s\" is not a structure",
-          compiler->last_error_extra_info);
-      break;
-    case ERROR_NOT_INDEXABLE:
-      snprintf(
-          buffer,
-          buffer_size,
-          "\"%s\" is not an array or dictionary",
-          compiler->last_error_extra_info);
-      break;
-    case ERROR_NOT_A_FUNCTION:
-      snprintf(
-          buffer,
-          buffer_size,
-          "\"%s\" is not a function",
-          compiler->last_error_extra_info);
-      break;
-    case ERROR_INVALID_FIELD_NAME:
-      snprintf(
-          buffer,
-          buffer_size,
-          "invalid field name \"%s\"",
-          compiler->last_error_extra_info);
-      break;
-    case ERROR_MISPLACED_ANONYMOUS_STRING:
-      snprintf(
-          buffer,
-          buffer_size,
-          "wrong use of anonymous string");
-      break;
-    case ERROR_INCLUDES_CIRCULAR_REFERENCE:
-      snprintf(
-          buffer,
-          buffer_size,
-          "include circular reference");
-      break;
-    case ERROR_INCLUDE_DEPTH_EXCEEDED:
-      snprintf(buffer,
-          buffer_size,
-          "too many levels of included rules");
-      break;
-    case ERROR_LOOP_NESTING_LIMIT_EXCEEDED:
-      snprintf(buffer,
-          buffer_size,
-          "loop nesting limit exceeded");
-      break;
-    case ERROR_NESTED_FOR_OF_LOOP:
-      snprintf(buffer,
-          buffer_size,
-          "'for <quantifier> of <string set>' loops can't be nested");
-      break;
-    case ERROR_UNKNOWN_MODULE:
-      snprintf(
-          buffer,
-          buffer_size,
-          "unknown module \"%s\"",
-          compiler->last_error_extra_info);
-      break;
-    case ERROR_INVALID_MODULE_NAME:
-      snprintf(
-          buffer,
-          buffer_size,
-          "invalid module name \"%s\"",
-          compiler->last_error_extra_info);
-      break;
-    case ERROR_DUPLICATED_STRUCTURE_MEMBER:
-      snprintf(buffer,
-          buffer_size,
-          "duplicated structure member");
-      break;
-    case ERROR_WRONG_ARGUMENTS:
-      snprintf(
-          buffer,
-          buffer_size,
-          "wrong arguments for function \"%s\"",
-          compiler->last_error_extra_info);
-      break;
-    case ERROR_WRONG_RETURN_TYPE:
-      snprintf(buffer,
-          buffer_size,
-          "wrong return type for overloaded function");
-      break;
-    case ERROR_INVALID_HEX_STRING:
-    case ERROR_INVALID_REGULAR_EXPRESSION:
-    case ERROR_SYNTAX_ERROR:
-    case ERROR_WRONG_TYPE:
-      snprintf(
-          buffer,
-          buffer_size,
-          "%s",
-          compiler->last_error_extra_info);
-      break;
-    case ERROR_INTERNAL_FATAL_ERROR:
-      snprintf(
-          buffer,
-          buffer_size,
-          "internal fatal error");
-      break;
-    case ERROR_DIVISION_BY_ZERO:
-      snprintf(
-          buffer,
-          buffer_size,
-          "division by zero");
-      break;
-    case ERROR_REGULAR_EXPRESSION_TOO_LARGE:
-      snprintf(
-          buffer,
-          buffer_size,
-          "regular expression is too large");
-      break;
-    case ERROR_REGULAR_EXPRESSION_TOO_COMPLEX:
-      snprintf(
-          buffer,
-          buffer_size,
-          "regular expression is too complex");
-      break;
-    case ERROR_TOO_MANY_STRINGS:
-       yr_get_configuration(
-          YR_CONFIG_MAX_STRINGS_PER_RULE,
-          &max_strings_per_rule);
-       snprintf(
-          buffer,
-          buffer_size,
-          "too many strings in rule \"%s\" (limit: %d)",
-          compiler->last_error_extra_info,
-          max_strings_per_rule);
-      break;
-    case ERROR_INTEGER_OVERFLOW:
-      snprintf(
-          buffer,
-          buffer_size,
-          "integer overflow in \"%s\"",
-          compiler->last_error_extra_info);
-      break;
-    case ERROR_COULD_NOT_READ_FILE:
-      snprintf(
-          buffer,
-          buffer_size,
-          "could not read file");
-      break;
-    case ERROR_INVALID_MODIFIER:
-      snprintf(
-          buffer,
-          buffer_size,
-          "invalid modifier combination \"%s\"",
-          compiler->last_error_extra_info);
-      break;
-    case ERROR_DUPLICATED_MODIFIER:
-      snprintf(
-          buffer,
-          buffer_size,
-          "duplicated modifier");
-      break;
+  case ERROR_INSUFFICIENT_MEMORY:
+    snprintf(buffer, buffer_size, "not enough memory");
+    break;
+  case ERROR_DUPLICATED_IDENTIFIER:
+    snprintf(
+        buffer,
+        buffer_size,
+        "duplicated identifier \"%s\"",
+        compiler->last_error_extra_info);
+    break;
+  case ERROR_DUPLICATED_STRING_IDENTIFIER:
+    snprintf(
+        buffer,
+        buffer_size,
+        "duplicated string identifier \"%s\"",
+        compiler->last_error_extra_info);
+    break;
+  case ERROR_DUPLICATED_TAG_IDENTIFIER:
+    snprintf(
+        buffer,
+        buffer_size,
+        "duplicated tag identifier \"%s\"",
+        compiler->last_error_extra_info);
+    break;
+  case ERROR_DUPLICATED_META_IDENTIFIER:
+    snprintf(
+        buffer,
+        buffer_size,
+        "duplicated metadata identifier \"%s\"",
+        compiler->last_error_extra_info);
+    break;
+  case ERROR_DUPLICATED_LOOP_IDENTIFIER:
+    snprintf(
+        buffer,
+        buffer_size,
+        "duplicated loop identifier \"%s\"",
+        compiler->last_error_extra_info);
+    break;
+  case ERROR_UNDEFINED_STRING:
+    snprintf(
+        buffer,
+        buffer_size,
+        "undefined string \"%s\"",
+        compiler->last_error_extra_info);
+    break;
+  case ERROR_UNDEFINED_IDENTIFIER:
+    snprintf(
+        buffer,
+        buffer_size,
+        "undefined identifier \"%s\"",
+        compiler->last_error_extra_info);
+    break;
+  case ERROR_UNREFERENCED_STRING:
+    snprintf(
+        buffer,
+        buffer_size,
+        "unreferenced string \"%s\"",
+        compiler->last_error_extra_info);
+    break;
+  case ERROR_EMPTY_STRING:
+    snprintf(
+        buffer,
+        buffer_size,
+        "empty string \"%s\"",
+        compiler->last_error_extra_info);
+    break;
+  case ERROR_NOT_A_STRUCTURE:
+    snprintf(
+        buffer,
+        buffer_size,
+        "\"%s\" is not a structure",
+        compiler->last_error_extra_info);
+    break;
+  case ERROR_NOT_INDEXABLE:
+    snprintf(
+        buffer,
+        buffer_size,
+        "\"%s\" is not an array or dictionary",
+        compiler->last_error_extra_info);
+    break;
+  case ERROR_NOT_A_FUNCTION:
+    snprintf(
+        buffer,
+        buffer_size,
+        "\"%s\" is not a function",
+        compiler->last_error_extra_info);
+    break;
+  case ERROR_INVALID_FIELD_NAME:
+    snprintf(
+        buffer,
+        buffer_size,
+        "invalid field name \"%s\"",
+        compiler->last_error_extra_info);
+    break;
+  case ERROR_MISPLACED_ANONYMOUS_STRING:
+    snprintf(buffer, buffer_size, "wrong use of anonymous string");
+    break;
+  case ERROR_INCLUDES_CIRCULAR_REFERENCE:
+    snprintf(buffer, buffer_size, "include circular reference");
+    break;
+  case ERROR_INCLUDE_DEPTH_EXCEEDED:
+    snprintf(buffer, buffer_size, "too many levels of included rules");
+    break;
+  case ERROR_LOOP_NESTING_LIMIT_EXCEEDED:
+    snprintf(buffer, buffer_size, "loop nesting limit exceeded");
+    break;
+  case ERROR_NESTED_FOR_OF_LOOP:
+    snprintf(
+        buffer,
+        buffer_size,
+        "'for <quantifier> of <string set>' loops can't be nested");
+    break;
+  case ERROR_UNKNOWN_MODULE:
+    snprintf(
+        buffer,
+        buffer_size,
+        "unknown module \"%s\"",
+        compiler->last_error_extra_info);
+    break;
+  case ERROR_INVALID_MODULE_NAME:
+    snprintf(
+        buffer,
+        buffer_size,
+        "invalid module name \"%s\"",
+        compiler->last_error_extra_info);
+    break;
+  case ERROR_DUPLICATED_STRUCTURE_MEMBER:
+    snprintf(buffer, buffer_size, "duplicated structure member");
+    break;
+  case ERROR_WRONG_ARGUMENTS:
+    snprintf(
+        buffer,
+        buffer_size,
+        "wrong arguments for function \"%s\"",
+        compiler->last_error_extra_info);
+    break;
+  case ERROR_WRONG_RETURN_TYPE:
+    snprintf(buffer, buffer_size, "wrong return type for overloaded function");
+    break;
+  case ERROR_INVALID_HEX_STRING:
+  case ERROR_INVALID_REGULAR_EXPRESSION:
+  case ERROR_SYNTAX_ERROR:
+  case ERROR_WRONG_TYPE:
+    snprintf(buffer, buffer_size, "%s", compiler->last_error_extra_info);
+    break;
+  case ERROR_INTERNAL_FATAL_ERROR:
+    snprintf(buffer, buffer_size, "internal fatal error");
+    break;
+  case ERROR_DIVISION_BY_ZERO:
+    snprintf(buffer, buffer_size, "division by zero");
+    break;
+  case ERROR_REGULAR_EXPRESSION_TOO_LARGE:
+    snprintf(buffer, buffer_size, "regular expression is too large");
+    break;
+  case ERROR_REGULAR_EXPRESSION_TOO_COMPLEX:
+    snprintf(buffer, buffer_size, "regular expression is too complex");
+    break;
+  case ERROR_TOO_MANY_STRINGS:
+    yr_get_configuration(YR_CONFIG_MAX_STRINGS_PER_RULE, &max_strings_per_rule);
+    snprintf(
+        buffer,
+        buffer_size,
+        "too many strings in rule \"%s\" (limit: %d)",
+        compiler->last_error_extra_info,
+        max_strings_per_rule);
+    break;
+  case ERROR_INTEGER_OVERFLOW:
+    snprintf(
+        buffer,
+        buffer_size,
+        "integer overflow in \"%s\"",
+        compiler->last_error_extra_info);
+    break;
+  case ERROR_COULD_NOT_READ_FILE:
+    snprintf(buffer, buffer_size, "could not read file");
+    break;
+  case ERROR_INVALID_MODIFIER:
+    snprintf(buffer, buffer_size, "%s", compiler->last_error_extra_info);
+    break;
+  case ERROR_DUPLICATED_MODIFIER:
+    snprintf(buffer, buffer_size, "duplicated modifier");
+    break;
   }
 
   return buffer;
