@@ -27,23 +27,20 @@ ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-
 #include <assert.h>
 #include <ctype.h>
+#include <math.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <math.h>
-
+#include <yara/error.h>
+#include <yara/exec.h>
 #include <yara/globals.h>
 #include <yara/mem.h>
-#include <yara/error.h>
 #include <yara/object.h>
-#include <yara/exec.h>
 #include <yara/utils.h>
 
-
 int yr_object_create(
     int8_t type,
     const char* identifier,
@@ -58,29 +55,29 @@ int yr_object_create(
 
   switch (type)
   {
-    case OBJECT_TYPE_STRUCTURE:
-      object_size = sizeof(YR_OBJECT_STRUCTURE);
-      break;
-    case OBJECT_TYPE_ARRAY:
-      object_size = sizeof(YR_OBJECT_ARRAY);
-      break;
-    case OBJECT_TYPE_DICTIONARY:
-      object_size = sizeof(YR_OBJECT_DICTIONARY);
-      break;
-    case OBJECT_TYPE_INTEGER:
-      object_size = sizeof(YR_OBJECT);
-      break;
-    case OBJECT_TYPE_FLOAT:
-      object_size = sizeof(YR_OBJECT);
-      break;
-    case OBJECT_TYPE_STRING:
-      object_size = sizeof(YR_OBJECT);
-      break;
-    case OBJECT_TYPE_FUNCTION:
-      object_size = sizeof(YR_OBJECT_FUNCTION);
-      break;
-    default:
-      assert(false);
+  case OBJECT_TYPE_STRUCTURE:
+    object_size = sizeof(YR_OBJECT_STRUCTURE);
+    break;
+  case OBJECT_TYPE_ARRAY:
+    object_size = sizeof(YR_OBJECT_ARRAY);
+    break;
+  case OBJECT_TYPE_DICTIONARY:
+    object_size = sizeof(YR_OBJECT_DICTIONARY);
+    break;
+  case OBJECT_TYPE_INTEGER:
+    object_size = sizeof(YR_OBJECT);
+    break;
+  case OBJECT_TYPE_FLOAT:
+    object_size = sizeof(YR_OBJECT);
+    break;
+  case OBJECT_TYPE_STRING:
+    object_size = sizeof(YR_OBJECT);
+    break;
+  case OBJECT_TYPE_FUNCTION:
+    object_size = sizeof(YR_OBJECT_FUNCTION);
+    break;
+  default:
+    assert(false);
   }
 
   obj = (YR_OBJECT*) yr_malloc(object_size);
@@ -93,36 +90,36 @@ int yr_object_create(
   obj->parent = parent;
   obj->data = NULL;
 
-  switch(type)
+  switch (type)
   {
-    case OBJECT_TYPE_INTEGER:
-      obj->value.i = YR_UNDEFINED;
-      break;
-    case OBJECT_TYPE_FLOAT:
-      obj->value.d = NAN;
-      break;
-    case OBJECT_TYPE_STRING:
-      obj->value.ss = NULL;
-      break;
-    case OBJECT_TYPE_STRUCTURE:
-      object_as_structure(obj)->members = NULL;
-      break;
-    case OBJECT_TYPE_ARRAY:
-      object_as_array(obj)->items = NULL;
-      object_as_array(obj)->prototype_item = NULL;
-      break;
-    case OBJECT_TYPE_DICTIONARY:
-      object_as_dictionary(obj)->items = NULL;
-      object_as_dictionary(obj)->prototype_item = NULL;
-      break;
-    case OBJECT_TYPE_FUNCTION:
-      object_as_function(obj)->return_obj = NULL;
-      for (i = 0; i < YR_MAX_OVERLOADED_FUNCTIONS; i++)
-      {
-        object_as_function(obj)->prototypes[i].arguments_fmt = NULL;
-        object_as_function(obj)->prototypes[i].code = NULL;
-      }
-      break;
+  case OBJECT_TYPE_INTEGER:
+    obj->value.i = YR_UNDEFINED;
+    break;
+  case OBJECT_TYPE_FLOAT:
+    obj->value.d = NAN;
+    break;
+  case OBJECT_TYPE_STRING:
+    obj->value.ss = NULL;
+    break;
+  case OBJECT_TYPE_STRUCTURE:
+    object_as_structure(obj)->members = NULL;
+    break;
+  case OBJECT_TYPE_ARRAY:
+    object_as_array(obj)->items = NULL;
+    object_as_array(obj)->prototype_item = NULL;
+    break;
+  case OBJECT_TYPE_DICTIONARY:
+    object_as_dictionary(obj)->items = NULL;
+    object_as_dictionary(obj)->prototype_item = NULL;
+    break;
+  case OBJECT_TYPE_FUNCTION:
+    object_as_function(obj)->return_obj = NULL;
+    for (i = 0; i < YR_MAX_OVERLOADED_FUNCTIONS; i++)
+    {
+      object_as_function(obj)->prototypes[i].arguments_fmt = NULL;
+      object_as_function(obj)->prototypes[i].code = NULL;
+    }
+    break;
   }
 
   if (obj->identifier == NULL)
@@ -133,36 +130,35 @@ int yr_object_create(
 
   if (parent != NULL)
   {
-    assert(parent->type == OBJECT_TYPE_STRUCTURE ||
-           parent->type == OBJECT_TYPE_ARRAY ||
-           parent->type == OBJECT_TYPE_DICTIONARY ||
-           parent->type == OBJECT_TYPE_FUNCTION);
+    assert(
+        parent->type == OBJECT_TYPE_STRUCTURE ||
+        parent->type == OBJECT_TYPE_ARRAY ||
+        parent->type == OBJECT_TYPE_DICTIONARY ||
+        parent->type == OBJECT_TYPE_FUNCTION);
 
     // Objects with a parent take the canary from it.
     obj->canary = parent->canary;
 
-    switch(parent->type)
+    switch (parent->type)
     {
-      case OBJECT_TYPE_STRUCTURE:
-        FAIL_ON_ERROR_WITH_CLEANUP(
-            yr_object_structure_set_member(parent, obj),
-            {
-              yr_free((void*) obj->identifier);
-              yr_free(obj);
-            });
-        break;
+    case OBJECT_TYPE_STRUCTURE:
+      FAIL_ON_ERROR_WITH_CLEANUP(yr_object_structure_set_member(parent, obj), {
+        yr_free((void*) obj->identifier);
+        yr_free(obj);
+      });
+      break;
 
-      case OBJECT_TYPE_ARRAY:
-        object_as_array(parent)->prototype_item = obj;
-        break;
+    case OBJECT_TYPE_ARRAY:
+      object_as_array(parent)->prototype_item = obj;
+      break;
 
-      case OBJECT_TYPE_DICTIONARY:
-        object_as_dictionary(parent)->prototype_item = obj;
-        break;
+    case OBJECT_TYPE_DICTIONARY:
+      object_as_dictionary(parent)->prototype_item = obj;
+      break;
 
-      case OBJECT_TYPE_FUNCTION:
-        object_as_function(parent)->return_obj = obj;
-        break;
+    case OBJECT_TYPE_FUNCTION:
+      object_as_function(parent)->return_obj = obj;
+      break;
     }
   }
 
@@ -172,15 +168,11 @@ int yr_object_create(
   return ERROR_SUCCESS;
 }
 
-
-void yr_object_set_canary(
-    YR_OBJECT* object,
-    int canary)
+void yr_object_set_canary(YR_OBJECT* object, int canary)
 {
   object->canary = canary;
 }
 
-
 int yr_object_function_create(
     const char* identifier,
     const char* arguments_fmt,
@@ -201,17 +193,17 @@ int yr_object_function_create(
 
   switch (*return_fmt)
   {
-    case 'i':
-      return_type = OBJECT_TYPE_INTEGER;
-      break;
-    case 's':
-      return_type = OBJECT_TYPE_STRING;
-      break;
-    case 'f':
-      return_type = OBJECT_TYPE_FLOAT;
-      break;
-    default:
-      return ERROR_INVALID_FORMAT;
+  case 'i':
+    return_type = OBJECT_TYPE_INTEGER;
+    break;
+  case 's':
+    return_type = OBJECT_TYPE_STRING;
+    break;
+  case 'f':
+    return_type = OBJECT_TYPE_FLOAT;
+    break;
+  default:
+    return ERROR_INVALID_FORMAT;
   }
 
   // Try to find if the structure already has a function
@@ -222,21 +214,13 @@ int yr_object_function_create(
   if (f != NULL && return_type != f->return_obj->type)
     return ERROR_WRONG_RETURN_TYPE;
 
-  if (f == NULL) // Function doesn't exist yet
+  if (f == NULL)  // Function doesn't exist yet
   {
     FAIL_ON_ERROR(
-        yr_object_create(
-            OBJECT_TYPE_FUNCTION,
-            identifier,
-            parent,
-            &o));
+        yr_object_create(OBJECT_TYPE_FUNCTION, identifier, parent, &o));
 
     FAIL_ON_ERROR_WITH_CLEANUP(
-        yr_object_create(
-            return_type,
-            "result",
-            o,
-            &return_obj),
+        yr_object_create(return_type, "result", o, &return_obj),
         yr_object_destroy(o));
 
     f = object_as_function(o);
@@ -259,7 +243,6 @@ int yr_object_function_create(
   return ERROR_SUCCESS;
 }
 
-
 int yr_object_from_external_variable(
     YR_EXTERNAL_VARIABLE* external,
     YR_OBJECT** object)
@@ -268,50 +251,46 @@ int yr_object_from_external_variable(
   int result;
   uint8_t obj_type = 0;
 
-  switch(external->type)
+  switch (external->type)
   {
+  case EXTERNAL_VARIABLE_TYPE_INTEGER:
+  case EXTERNAL_VARIABLE_TYPE_BOOLEAN:
+    obj_type = OBJECT_TYPE_INTEGER;
+    break;
+
+  case EXTERNAL_VARIABLE_TYPE_FLOAT:
+    obj_type = OBJECT_TYPE_FLOAT;
+    break;
+
+  case EXTERNAL_VARIABLE_TYPE_STRING:
+  case EXTERNAL_VARIABLE_TYPE_MALLOC_STRING:
+    obj_type = OBJECT_TYPE_STRING;
+    break;
+
+  default:
+    assert(false);
+  }
+
+  result = yr_object_create(obj_type, external->identifier, NULL, &obj);
+
+  if (result == ERROR_SUCCESS)
+  {
+    switch (external->type)
+    {
     case EXTERNAL_VARIABLE_TYPE_INTEGER:
     case EXTERNAL_VARIABLE_TYPE_BOOLEAN:
-      obj_type = OBJECT_TYPE_INTEGER;
+      result = yr_object_set_integer(external->value.i, obj, NULL);
       break;
 
     case EXTERNAL_VARIABLE_TYPE_FLOAT:
-      obj_type = OBJECT_TYPE_FLOAT;
+      result = yr_object_set_float(external->value.f, obj, NULL);
       break;
 
     case EXTERNAL_VARIABLE_TYPE_STRING:
     case EXTERNAL_VARIABLE_TYPE_MALLOC_STRING:
-      obj_type = OBJECT_TYPE_STRING;
+      result = yr_object_set_string(
+          external->value.s, strlen(external->value.s), obj, NULL);
       break;
-
-    default:
-      assert(false);
-  }
-
-  result = yr_object_create(
-      obj_type,
-      external->identifier,
-      NULL,
-      &obj);
-
-  if (result == ERROR_SUCCESS)
-  {
-    switch(external->type)
-    {
-      case EXTERNAL_VARIABLE_TYPE_INTEGER:
-      case EXTERNAL_VARIABLE_TYPE_BOOLEAN:
-        result = yr_object_set_integer(external->value.i, obj, NULL);
-        break;
-
-      case EXTERNAL_VARIABLE_TYPE_FLOAT:
-        result = yr_object_set_float(external->value.f, obj, NULL);
-        break;
-
-      case EXTERNAL_VARIABLE_TYPE_STRING:
-      case EXTERNAL_VARIABLE_TYPE_MALLOC_STRING:
-        result = yr_object_set_string(
-            external->value.s, strlen(external->value.s), obj, NULL);
-        break;
     }
 
     if (result == ERROR_SUCCESS)
@@ -327,9 +306,7 @@ int yr_object_from_external_variable(
   return result;
 }
 
-
-void yr_object_destroy(
-    YR_OBJECT* object)
+void yr_object_destroy(YR_OBJECT* object)
 {
   YR_STRUCTURE_MEMBER* member;
   YR_STRUCTURE_MEMBER* next_member;
@@ -341,75 +318,72 @@ void yr_object_destroy(
   if (object == NULL)
     return;
 
-  switch(object->type)
+  switch (object->type)
   {
-    case OBJECT_TYPE_STRUCTURE:
-      member = object_as_structure(object)->members;
+  case OBJECT_TYPE_STRUCTURE:
+    member = object_as_structure(object)->members;
 
-      while (member != NULL)
+    while (member != NULL)
+    {
+      next_member = member->next;
+      yr_object_destroy(member->object);
+      yr_free(member);
+      member = next_member;
+    }
+    break;
+
+  case OBJECT_TYPE_STRING:
+    if (object->value.ss != NULL)
+      yr_free(object->value.ss);
+    break;
+
+  case OBJECT_TYPE_ARRAY:
+    if (object_as_array(object)->prototype_item != NULL)
+      yr_object_destroy(object_as_array(object)->prototype_item);
+
+    array_items = object_as_array(object)->items;
+
+    if (array_items != NULL)
+    {
+      for (i = 0; i < array_items->length; i++)
+        if (array_items->objects[i] != NULL)
+          yr_object_destroy(array_items->objects[i]);
+    }
+
+    yr_free(array_items);
+    break;
+
+  case OBJECT_TYPE_DICTIONARY:
+    if (object_as_dictionary(object)->prototype_item != NULL)
+      yr_object_destroy(object_as_dictionary(object)->prototype_item);
+
+    dict_items = object_as_dictionary(object)->items;
+
+    if (dict_items != NULL)
+    {
+      for (i = 0; i < dict_items->used; i++)
       {
-        next_member = member->next;
-        yr_object_destroy(member->object);
-        yr_free(member);
-        member = next_member;
+        if (dict_items->objects[i].key != NULL)
+          yr_free(dict_items->objects[i].key);
+
+        if (dict_items->objects[i].obj != NULL)
+          yr_object_destroy(dict_items->objects[i].obj);
       }
-      break;
+    }
 
-    case OBJECT_TYPE_STRING:
-      if (object->value.ss != NULL)
-        yr_free(object->value.ss);
-      break;
+    yr_free(dict_items);
+    break;
 
-    case OBJECT_TYPE_ARRAY:
-      if (object_as_array(object)->prototype_item != NULL)
-        yr_object_destroy(object_as_array(object)->prototype_item);
-
-      array_items = object_as_array(object)->items;
-
-      if (array_items != NULL)
-      {
-        for (i = 0; i < array_items->length; i++)
-          if (array_items->objects[i] != NULL)
-            yr_object_destroy(array_items->objects[i]);
-      }
-
-      yr_free(array_items);
-      break;
-
-    case OBJECT_TYPE_DICTIONARY:
-      if (object_as_dictionary(object)->prototype_item != NULL)
-        yr_object_destroy(object_as_dictionary(object)->prototype_item);
-
-      dict_items = object_as_dictionary(object)->items;
-
-      if (dict_items != NULL)
-      {
-        for (i = 0; i < dict_items->used; i++)
-        {
-          if (dict_items->objects[i].key != NULL)
-            yr_free(dict_items->objects[i].key);
-
-          if (dict_items->objects[i].obj != NULL)
-            yr_object_destroy(dict_items->objects[i].obj);
-        }
-      }
-
-      yr_free(dict_items);
-      break;
-
-    case OBJECT_TYPE_FUNCTION:
-      yr_object_destroy(object_as_function(object)->return_obj);
-      break;
+  case OBJECT_TYPE_FUNCTION:
+    yr_object_destroy(object_as_function(object)->return_obj);
+    break;
   }
 
   yr_free((void*) object->identifier);
   yr_free(object);
 }
 
-
-YR_OBJECT* yr_object_lookup_field(
-    YR_OBJECT* object,
-    const char* field_name)
+YR_OBJECT* yr_object_lookup_field(YR_OBJECT* object, const char* field_name)
 {
   YR_STRUCTURE_MEMBER* member;
 
@@ -429,7 +403,6 @@ YR_OBJECT* yr_object_lookup_field(
   return NULL;
 }
 
-
 static YR_OBJECT* _yr_object_lookup(
     YR_OBJECT* object,
     int flags,
@@ -473,17 +446,17 @@ static YR_OBJECT* _yr_object_lookup(
       {
         p++;
 
-        switch(*p++)
+        switch (*p++)
         {
-          case 'i':
-            index = va_arg(args, int);
-            break;
-          case 's':
-            key = va_arg(args, const char*);
-            break;
+        case 'i':
+          index = va_arg(args, int);
+          break;
+        case 's':
+          key = va_arg(args, const char*);
+          break;
 
-          default:
-            return NULL;
+        default:
+          return NULL;
         }
       }
       else if (*p >= '0' && *p <= '9')
@@ -493,13 +466,12 @@ static YR_OBJECT* _yr_object_lookup(
       else if (*p == '"')
       {
         i = 0;
-        p++;              // skip the opening quotation mark
+        p++;  // skip the opening quotation mark
 
-        while (*p != '"' && *p != '\0' && i < sizeof(str) - 1)
-          str[i++] = *p++;
+        while (*p != '"' && *p != '\0' && i < sizeof(str) - 1) str[i++] = *p++;
 
         str[i] = '\0';
-        p++;              // skip the closing quotation mark
+        p++;  // skip the closing quotation mark
         key = str;
       }
       else
@@ -511,17 +483,17 @@ static YR_OBJECT* _yr_object_lookup(
       p++;
       assert(*p == '.' || *p == '\0');
 
-      switch(obj->type)
+      switch (obj->type)
       {
-        case OBJECT_TYPE_ARRAY:
-          assert(index != -1);
-          obj = yr_object_array_get_item(obj, flags, index);
-          break;
+      case OBJECT_TYPE_ARRAY:
+        assert(index != -1);
+        obj = yr_object_array_get_item(obj, flags, index);
+        break;
 
-        case OBJECT_TYPE_DICTIONARY:
-          assert(key != NULL);
-          obj = yr_object_dict_get_item(obj, flags, key);
-          break;
+      case OBJECT_TYPE_DICTIONARY:
+        assert(key != NULL);
+        obj = yr_object_dict_get_item(obj, flags, key);
+        break;
       }
     }
 
@@ -534,7 +506,6 @@ static YR_OBJECT* _yr_object_lookup(
   return obj;
 }
 
-
 YR_OBJECT* yr_object_lookup(
     YR_OBJECT* object,
     int flags,
@@ -551,13 +522,9 @@ YR_OBJECT* yr_object_lookup(
   va_end(args);
 
   return result;
-
 }
 
-
-int yr_object_copy(
-    YR_OBJECT* object,
-    YR_OBJECT** object_copy)
+int yr_object_copy(YR_OBJECT* object, YR_OBJECT** object_copy)
 {
   YR_OBJECT* copy;
   YR_OBJECT* o;
@@ -568,92 +535,87 @@ int yr_object_copy(
 
   *object_copy = NULL;
 
-  FAIL_ON_ERROR(yr_object_create(
-      object->type,
-      object->identifier,
-      NULL,
-      &copy));
+  FAIL_ON_ERROR(
+      yr_object_create(object->type, object->identifier, NULL, &copy));
 
   copy->canary = object->canary;
 
-  switch(object->type)
+  switch (object->type)
   {
-    case OBJECT_TYPE_INTEGER:
-      copy->value.i = object->value.i;
-      break;
+  case OBJECT_TYPE_INTEGER:
+    copy->value.i = object->value.i;
+    break;
 
-    case OBJECT_TYPE_FLOAT:
-      copy->value.d = object->value.d;
-      break;
+  case OBJECT_TYPE_FLOAT:
+    copy->value.d = object->value.d;
+    break;
 
-    case OBJECT_TYPE_STRING:
+  case OBJECT_TYPE_STRING:
 
-      if (object->value.ss != NULL)
-        copy->value.ss = sized_string_dup(object->value.ss);
-      else
-        copy->value.ss = NULL;
+    if (object->value.ss != NULL)
+      copy->value.ss = ss_dup(object->value.ss);
+    else
+      copy->value.ss = NULL;
 
-      break;
+    break;
 
-    case OBJECT_TYPE_FUNCTION:
+  case OBJECT_TYPE_FUNCTION:
 
+    FAIL_ON_ERROR_WITH_CLEANUP(
+        yr_object_copy(
+            object_as_function(object)->return_obj,
+            &object_as_function(copy)->return_obj),
+        // cleanup
+        yr_object_destroy(copy));
+
+    for (i = 0; i < YR_MAX_OVERLOADED_FUNCTIONS; i++)
+      object_as_function(copy)->prototypes[i] =
+          object_as_function(object)->prototypes[i];
+
+    break;
+
+  case OBJECT_TYPE_STRUCTURE:
+
+    structure_member = object_as_structure(object)->members;
+
+    while (structure_member != NULL)
+    {
       FAIL_ON_ERROR_WITH_CLEANUP(
-          yr_object_copy(
-              object_as_function(object)->return_obj,
-              &object_as_function(copy)->return_obj),
-          // cleanup
+          yr_object_copy(structure_member->object, &o),
           yr_object_destroy(copy));
 
-      for (i = 0; i < YR_MAX_OVERLOADED_FUNCTIONS; i++)
-        object_as_function(copy)->prototypes[i] = \
-            object_as_function(object)->prototypes[i];
+      FAIL_ON_ERROR_WITH_CLEANUP(yr_object_structure_set_member(copy, o),
+                                 // cleanup
+                                 yr_free(o);
+                                 yr_object_destroy(copy));
 
-      break;
+      structure_member = structure_member->next;
+    }
 
-    case OBJECT_TYPE_STRUCTURE:
+    break;
 
-      structure_member = object_as_structure(object)->members;
+  case OBJECT_TYPE_ARRAY:
 
-      while (structure_member != NULL)
-      {
-        FAIL_ON_ERROR_WITH_CLEANUP(
-            yr_object_copy(structure_member->object, &o),
-            yr_object_destroy(copy));
+    FAIL_ON_ERROR_WITH_CLEANUP(
+        yr_object_copy(object_as_array(object)->prototype_item, &o),
+        yr_object_destroy(copy));
 
-        FAIL_ON_ERROR_WITH_CLEANUP(
-            yr_object_structure_set_member(copy, o),
-            // cleanup
-            yr_free(o);
-            yr_object_destroy(copy));
+    object_as_array(copy)->prototype_item = o;
 
-        structure_member = structure_member->next;
-      }
+    break;
 
-      break;
+  case OBJECT_TYPE_DICTIONARY:
 
-    case OBJECT_TYPE_ARRAY:
+    FAIL_ON_ERROR_WITH_CLEANUP(
+        yr_object_copy(object_as_dictionary(object)->prototype_item, &o),
+        yr_object_destroy(copy));
 
-      FAIL_ON_ERROR_WITH_CLEANUP(
-          yr_object_copy(object_as_array(object)->prototype_item, &o),
-          yr_object_destroy(copy));
+    object_as_dictionary(copy)->prototype_item = o;
 
-      object_as_array(copy)->prototype_item = o;
-
-      break;
-
-    case OBJECT_TYPE_DICTIONARY:
-
-      FAIL_ON_ERROR_WITH_CLEANUP(
-          yr_object_copy(object_as_dictionary(object)->prototype_item, &o),
-          yr_object_destroy(copy));
-
-      object_as_dictionary(copy)->prototype_item = o;
-
-      break;
-
-    default:
-      assert(false);
+    break;
 
+  default:
+    assert(false);
   }
 
   *object_copy = copy;
@@ -661,10 +623,7 @@ int yr_object_copy(
   return ERROR_SUCCESS;
 }
 
-
-int yr_object_structure_set_member(
-    YR_OBJECT* object,
-    YR_OBJECT* member)
+int yr_object_structure_set_member(YR_OBJECT* object, YR_OBJECT* member)
 {
   YR_STRUCTURE_MEMBER* sm;
 
@@ -672,7 +631,7 @@ int yr_object_structure_set_member(
 
   // Check if the object already have a member with the same identifier
 
-  if (yr_object_lookup_field(object,  member->identifier) != NULL)
+  if (yr_object_lookup_field(object, member->identifier) != NULL)
     return ERROR_DUPLICATED_STRUCTURE_MEMBER;
 
   sm = (YR_STRUCTURE_MEMBER*) yr_malloc(sizeof(YR_STRUCTURE_MEMBER));
@@ -689,9 +648,7 @@ int yr_object_structure_set_member(
   return ERROR_SUCCESS;
 }
 
-
-int yr_object_array_length(
-    YR_OBJECT* object)
+int yr_object_array_length(YR_OBJECT* object)
 {
   YR_OBJECT_ARRAY* array;
 
@@ -704,11 +661,7 @@ int yr_object_array_length(
   return array->items->length;
 }
 
-
-YR_OBJECT* yr_object_array_get_item(
-    YR_OBJECT* object,
-    int flags,
-    int index)
+YR_OBJECT* yr_object_array_get_item(YR_OBJECT* object, int flags, int index)
 {
   YR_OBJECT* result = NULL;
   YR_OBJECT_ARRAY* array;
@@ -734,11 +687,7 @@ YR_OBJECT* yr_object_array_get_item(
   return result;
 }
 
-
-int yr_object_array_set_item(
-    YR_OBJECT* object,
-    YR_OBJECT* item,
-    int index)
+int yr_object_array_set_item(YR_OBJECT* object, YR_OBJECT* item, int index)
 {
   YR_OBJECT_ARRAY* array;
 
@@ -754,8 +703,7 @@ int yr_object_array_set_item(
   {
     capacity = 64;
 
-    while (capacity <= index)
-      capacity *= 2;
+    while (capacity <= index) capacity *= 2;
 
     array->items = (YR_ARRAY_ITEMS*) yr_malloc(
         sizeof(YR_ARRAY_ITEMS) + capacity * sizeof(YR_OBJECT*));
@@ -772,12 +720,10 @@ int yr_object_array_set_item(
   {
     capacity = array->items->capacity * 2;
 
-    while (capacity <= index)
-      capacity *= 2;
+    while (capacity <= index) capacity *= 2;
 
     array->items = (YR_ARRAY_ITEMS*) yr_realloc(
-        array->items,
-        sizeof(YR_ARRAY_ITEMS) + capacity * sizeof(YR_OBJECT*));
+        array->items, sizeof(YR_ARRAY_ITEMS) + capacity * sizeof(YR_OBJECT*));
 
     if (array->items == NULL)
       return ERROR_INSUFFICIENT_MEMORY;
@@ -797,7 +743,6 @@ int yr_object_array_set_item(
   return ERROR_SUCCESS;
 }
 
-
 YR_OBJECT* yr_object_dict_get_item(
     YR_OBJECT* object,
     int flags,
@@ -832,11 +777,7 @@ YR_OBJECT* yr_object_dict_get_item(
   return result;
 }
 
-
-int yr_object_dict_set_item(
-    YR_OBJECT* object,
-    YR_OBJECT* item,
-    const char* key)
+int yr_object_dict_set_item(YR_OBJECT* object, YR_OBJECT* item, const char* key)
 {
   YR_OBJECT_DICTIONARY* dict;
 
@@ -883,7 +824,7 @@ int yr_object_dict_set_item(
 
   item->parent = object;
 
-  dict->items->objects[dict->items->used].key = sized_string_new(key);
+  dict->items->objects[dict->items->used].key = ss_new(key);
   dict->items->objects[dict->items->used].obj = item;
 
   dict->items->used++;
@@ -892,11 +833,7 @@ int yr_object_dict_set_item(
   return ERROR_SUCCESS;
 }
 
-
-bool yr_object_has_undefined_value(
-    YR_OBJECT* object,
-    const char* field,
-    ...)
+bool yr_object_has_undefined_value(YR_OBJECT* object, const char* field, ...)
 {
   YR_OBJECT* field_obj;
 
@@ -913,24 +850,20 @@ bool yr_object_has_undefined_value(
   if (field_obj == NULL)
     return true;
 
-  switch(field_obj->type)
+  switch (field_obj->type)
   {
-    case OBJECT_TYPE_FLOAT:
-      return isnan(field_obj->value.d);
-    case OBJECT_TYPE_STRING:
-      return field_obj->value.ss == NULL;
-    case OBJECT_TYPE_INTEGER:
-      return field_obj->value.i == YR_UNDEFINED;
+  case OBJECT_TYPE_FLOAT:
+    return isnan(field_obj->value.d);
+  case OBJECT_TYPE_STRING:
+    return field_obj->value.ss == NULL;
+  case OBJECT_TYPE_INTEGER:
+    return field_obj->value.i == YR_UNDEFINED;
   }
 
   return false;
 }
 
-
-int64_t yr_object_get_integer(
-    YR_OBJECT* object,
-    const char* field,
-    ...)
+int64_t yr_object_get_integer(YR_OBJECT* object, const char* field, ...)
 {
   YR_OBJECT* integer_obj;
 
@@ -947,17 +880,15 @@ int64_t yr_object_get_integer(
   if (integer_obj == NULL)
     return YR_UNDEFINED;
 
-  assertf(integer_obj->type == OBJECT_TYPE_INTEGER,
-          "type of \"%s\" is not integer\n", field);
+  assertf(
+      integer_obj->type == OBJECT_TYPE_INTEGER,
+      "type of \"%s\" is not integer\n",
+      field);
 
   return integer_obj->value.i;
 }
 
-
-double yr_object_get_float(
-    YR_OBJECT* object,
-    const char* field,
-    ...)
+double yr_object_get_float(YR_OBJECT* object, const char* field, ...)
 {
   YR_OBJECT* double_obj;
 
@@ -974,17 +905,15 @@ double yr_object_get_float(
   if (double_obj == NULL)
     return NAN;
 
-  assertf(double_obj->type == OBJECT_TYPE_FLOAT,
-          "type of \"%s\" is not double\n", field);
+  assertf(
+      double_obj->type == OBJECT_TYPE_FLOAT,
+      "type of \"%s\" is not double\n",
+      field);
 
   return double_obj->value.d;
 }
 
-
-SIZED_STRING* yr_object_get_string(
-    YR_OBJECT* object,
-    const char* field,
-    ...)
+SIZED_STRING* yr_object_get_string(YR_OBJECT* object, const char* field, ...)
 {
   YR_OBJECT* string_obj;
 
@@ -1001,13 +930,14 @@ SIZED_STRING* yr_object_get_string(
   if (string_obj == NULL)
     return NULL;
 
-  assertf(string_obj->type == OBJECT_TYPE_STRING,
-          "type of \"%s\" is not string\n", field);
+  assertf(
+      string_obj->type == OBJECT_TYPE_STRING,
+      "type of \"%s\" is not string\n",
+      field);
 
   return string_obj->value.ss;
 }
 
-
 int yr_object_set_integer(
     int64_t value,
     YR_OBJECT* object,
@@ -1041,12 +971,7 @@ int yr_object_set_integer(
   return ERROR_SUCCESS;
 }
 
-
-int yr_object_set_float(
-    double value,
-    YR_OBJECT* object,
-    const char* field,
-    ...)
+int yr_object_set_float(double value, YR_OBJECT* object, const char* field, ...)
 {
   YR_OBJECT* double_obj;
 
@@ -1075,7 +1000,6 @@ int yr_object_set_float(
   return ERROR_SUCCESS;
 }
 
-
 int yr_object_set_string(
     const char* value,
     size_t len,
@@ -1130,14 +1054,11 @@ int yr_object_set_string(
   return ERROR_SUCCESS;
 }
 
-
-YR_OBJECT* yr_object_get_root(
-    YR_OBJECT* object)
+YR_OBJECT* yr_object_get_root(YR_OBJECT* object)
 {
   YR_OBJECT* o = object;
 
-  while (o->parent != NULL)
-    o = o->parent;
+  while (o->parent != NULL) o = o->parent;
 
   return o;
 }
@@ -1161,97 +1082,94 @@ YR_API void yr_object_print_data(
   if (print_identifier && object->type != OBJECT_TYPE_FUNCTION)
     printf("%s%s", indent_spaces, object->identifier);
 
-  switch(object->type)
+  switch (object->type)
   {
-    case OBJECT_TYPE_FLOAT:
-      if (object->value.i != YR_UNDEFINED)
-        printf(" = %f", object->value.d);
-      else
-        printf(" = YR_UNDEFINED");
+  case OBJECT_TYPE_FLOAT:
+    if (object->value.i != YR_UNDEFINED)
+      printf(" = %f", object->value.d);
+    else
+      printf(" = YR_UNDEFINED");
 
-      break;
+    break;
 
-    case OBJECT_TYPE_INTEGER:
+  case OBJECT_TYPE_INTEGER:
 
-      if (object->value.i != YR_UNDEFINED)
-        printf(" = %" PRId64, object->value.i);
-      else
-        printf(" = YR_UNDEFINED");
+    if (object->value.i != YR_UNDEFINED)
+      printf(" = %" PRId64, object->value.i);
+    else
+      printf(" = YR_UNDEFINED");
 
-      break;
+    break;
 
-    case OBJECT_TYPE_STRING:
+  case OBJECT_TYPE_STRING:
 
-      if (object->value.ss != NULL)
+    if (object->value.ss != NULL)
+    {
+      size_t l;
+      printf(" = \"");
+
+      for (l = 0; l < object->value.ss->length; l++)
       {
-        size_t l;
-        printf(" = \"");
+        char c = object->value.ss->c_string[l];
 
-        for (l = 0; l < object->value.ss->length; l++)
-        {
-          char c = object->value.ss->c_string[l];
-
-          if (isprint((unsigned char) c))
-            printf("%c", c);
-          else
-            printf("\\x%02x", (unsigned char) c);
-        }
-
-        printf("\"");
-      }
-      else
-      {
-        printf(" = YR_UNDEFINED");
+        if (isprint((unsigned char) c))
+          printf("%c", c);
+        else
+          printf("\\x%02x", (unsigned char) c);
       }
 
-      break;
+      printf("\"");
+    }
+    else
+    {
+      printf(" = YR_UNDEFINED");
+    }
 
-    case OBJECT_TYPE_STRUCTURE:
+    break;
 
-      member = object_as_structure(object)->members;
+  case OBJECT_TYPE_STRUCTURE:
 
-      while (member != NULL)
+    member = object_as_structure(object)->members;
+
+    while (member != NULL)
+    {
+      if (member->object->type != OBJECT_TYPE_FUNCTION)
       {
-        if (member->object->type != OBJECT_TYPE_FUNCTION)
-        {
-          printf("\n");
-          yr_object_print_data(member->object, indent + 1, 1);
-        }
-        member = member->next;
+        printf("\n");
+        yr_object_print_data(member->object, indent + 1, 1);
       }
+      member = member->next;
+    }
 
-      break;
+    break;
 
-    case OBJECT_TYPE_ARRAY:
-      for (i = 0; i < yr_object_array_length(object); i++)
+  case OBJECT_TYPE_ARRAY:
+    for (i = 0; i < yr_object_array_length(object); i++)
+    {
+      YR_OBJECT* o = yr_object_array_get_item(object, 0, i);
+
+      if (o != NULL)
       {
-        YR_OBJECT* o = yr_object_array_get_item(object, 0, i);
-
-        if (o != NULL)
-        {
-          printf("\n%s\t[%d]", indent_spaces, i);
-          yr_object_print_data(o, indent + 1, 0);
-        }
+        printf("\n%s\t[%d]", indent_spaces, i);
+        yr_object_print_data(o, indent + 1, 0);
       }
-      break;
+    }
+    break;
 
-    case OBJECT_TYPE_DICTIONARY:
+  case OBJECT_TYPE_DICTIONARY:
 
-      dict_items = object_as_dictionary(object)->items;
+    dict_items = object_as_dictionary(object)->items;
 
-      if (dict_items != NULL)
+    if (dict_items != NULL)
+    {
+      for (i = 0; i < dict_items->used; i++)
       {
-        for (i = 0; i < dict_items->used; i++)
-        {
-          printf(
-              "\n%s\t%s",
-              indent_spaces,
-              dict_items->objects[i].key->c_string);
+        printf("\n%s\t%s", indent_spaces, dict_items->objects[i].key->c_string);
 
-          yr_object_print_data(dict_items->objects[i].obj, indent + 1, 0);
-        }
+        yr_object_print_data(dict_items->objects[i].obj, indent + 1, 0);
       }
+    }
 
-      break;
+    break;
   }
 }
