@@ -30,20 +30,18 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #if defined(USE_WINDOWS_PROC)
 
 #include <windows.h>
-
-#include <yara/mem.h>
 #include <yara/error.h>
+#include <yara/mem.h>
 #include <yara/proc.h>
 
 
-typedef struct _YR_PROC_INFO {
-  HANDLE          hProcess;
-  SYSTEM_INFO     si;
+typedef struct _YR_PROC_INFO
+{
+  HANDLE hProcess;
+  SYSTEM_INFO si;
 } YR_PROC_INFO;
 
-int _yr_process_attach(
-    int pid,
-    YR_PROC_ITERATOR_CTX* context)
+int _yr_process_attach(int pid, YR_PROC_ITERATOR_CTX* context)
 {
   TOKEN_PRIVILEGES tokenPriv;
   LUID luidDebug;
@@ -62,21 +60,14 @@ int _yr_process_attach(
     tokenPriv.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
 
     AdjustTokenPrivileges(
-        hToken,
-        FALSE,
-        &tokenPriv,
-        sizeof(tokenPriv),
-        NULL,
-        NULL);
+        hToken, FALSE, &tokenPriv, sizeof(tokenPriv), NULL, NULL);
   }
 
   if (hToken != NULL)
     CloseHandle(hToken);
 
   proc_info->hProcess = OpenProcess(
-      PROCESS_VM_READ | PROCESS_QUERY_INFORMATION,
-      FALSE,
-      pid);
+      PROCESS_VM_READ | PROCESS_QUERY_INFORMATION, FALSE, pid);
 
   if (proc_info->hProcess == NULL)
   {
@@ -92,18 +83,16 @@ int _yr_process_attach(
 }
 
 
-int _yr_process_detach(
-    YR_PROC_ITERATOR_CTX* context)
+int _yr_process_detach(YR_PROC_ITERATOR_CTX* context)
 {
-  YR_PROC_INFO* proc_info = (YR_PROC_INFO*)context->proc_info;
+  YR_PROC_INFO* proc_info = (YR_PROC_INFO*) context->proc_info;
 
   CloseHandle(proc_info->hProcess);
   return ERROR_SUCCESS;
 }
 
 
-YR_API const uint8_t* yr_process_fetch_memory_block_data(
-    YR_MEMORY_BLOCK* block)
+YR_API const uint8_t* yr_process_fetch_memory_block_data(YR_MEMORY_BLOCK* block)
 {
   SIZE_T read;
 
@@ -129,14 +118,14 @@ YR_API const uint8_t* yr_process_fetch_memory_block_data(
   }
 
   if (ReadProcessMemory(
-        proc_info->hProcess,
-        (LPCVOID) block->base,
-        (LPVOID) context->buffer,
-        (SIZE_T) block->size,
-        &read) == FALSE)
-    {
-      return NULL;
-    }
+          proc_info->hProcess,
+          (LPCVOID) block->base,
+          (LPVOID) context->buffer,
+          (SIZE_T) block->size,
+          &read) == FALSE)
+  {
+    return NULL;
+  }
 
   return context->buffer;
 }
@@ -149,8 +138,10 @@ YR_API YR_MEMORY_BLOCK* yr_process_get_next_memory_block(
   YR_PROC_INFO* proc_info = (YR_PROC_INFO*) context->proc_info;
 
   MEMORY_BASIC_INFORMATION mbi;
-  PVOID address = (PVOID) (context->current_block.base + \
-                           context->current_block.size);
+  PVOID address = (PVOID)(
+      context->current_block.base + context->current_block.size);
+
+  iterator->last_error = ERROR_SUCCESS;
 
   while (address < proc_info->si.lpMaximumApplicationAddress &&
          VirtualQueryEx(proc_info->hProcess, address, &mbi, sizeof(mbi)) != 0)
@@ -182,8 +173,8 @@ YR_API YR_MEMORY_BLOCK* yr_process_get_first_memory_block(
   YR_PROC_INFO* proc_info = (YR_PROC_INFO*) context->proc_info;
 
   context->current_block.size = 0;
-  context->current_block.base = \
-      (size_t) proc_info->si.lpMinimumApplicationAddress;
+  context->current_block.base = (size_t)
+                                    proc_info->si.lpMinimumApplicationAddress;
 
   return yr_process_get_next_memory_block(iterator);
 }
