@@ -28,20 +28,18 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
 #include <assert.h>
-#include <string.h>
 #include <ctype.h>
-
+#include <string.h>
+#include <yara/compiler.h>
 #include <yara/error.h>
 #include <yara/filemap.h>
+#include <yara/globals.h>
 #include <yara/mem.h>
 #include <yara/proc.h>
 #include <yara/rules.h>
-#include <yara/utils.h>
-#include <yara/globals.h>
 #include <yara/scan.h>
 #include <yara/scanner.h>
-#include <yara/compiler.h>
-
+#include <yara/utils.h>
 
 YR_API int yr_rules_define_integer_variable(
     YR_RULES* rules,
@@ -53,7 +51,7 @@ YR_API int yr_rules_define_integer_variable(
   if (identifier == NULL)
     return ERROR_INVALID_ARGUMENT;
 
-  external = rules->externals_list_head;
+  external = rules->ext_vars_table;
 
   while (!EXTERNAL_VARIABLE_IS_NULL(external))
   {
@@ -72,7 +70,6 @@ YR_API int yr_rules_define_integer_variable(
   return ERROR_INVALID_ARGUMENT;
 }
 
-
 YR_API int yr_rules_define_boolean_variable(
     YR_RULES* rules,
     const char* identifier,
@@ -83,7 +80,7 @@ YR_API int yr_rules_define_boolean_variable(
   if (identifier == NULL)
     return ERROR_INVALID_ARGUMENT;
 
-  external = rules->externals_list_head;
+  external = rules->ext_vars_table;
 
   while (!EXTERNAL_VARIABLE_IS_NULL(external))
   {
@@ -102,7 +99,6 @@ YR_API int yr_rules_define_boolean_variable(
   return ERROR_INVALID_ARGUMENT;
 }
 
-
 YR_API int yr_rules_define_float_variable(
     YR_RULES* rules,
     const char* identifier,
@@ -113,7 +109,7 @@ YR_API int yr_rules_define_float_variable(
   if (identifier == NULL)
     return ERROR_INVALID_ARGUMENT;
 
-  external = rules->externals_list_head;
+  external = rules->ext_vars_table;
 
   while (!EXTERNAL_VARIABLE_IS_NULL(external))
   {
@@ -132,7 +128,6 @@ YR_API int yr_rules_define_float_variable(
   return ERROR_INVALID_ARGUMENT;
 }
 
-
 YR_API int yr_rules_define_string_variable(
     YR_RULES* rules,
     const char* identifier,
@@ -143,7 +138,7 @@ YR_API int yr_rules_define_string_variable(
   if (identifier == NULL || value == NULL)
     return ERROR_INVALID_ARGUMENT;
 
-  external = rules->externals_list_head;
+  external = rules->ext_vars_table;
 
   while (!EXTERNAL_VARIABLE_IS_NULL(external))
   {
@@ -174,7 +169,6 @@ YR_API int yr_rules_define_string_variable(
   return ERROR_INVALID_ARGUMENT;
 }
 
-
 YR_API int yr_rules_scan_mem_blocks(
     YR_RULES* rules,
     YR_MEMORY_BLOCK_ITERATOR* iterator,
@@ -199,28 +193,6 @@ YR_API int yr_rules_scan_mem_blocks(
   return result;
 }
 
-
-static YR_MEMORY_BLOCK* _yr_get_first_block(
-    YR_MEMORY_BLOCK_ITERATOR* iterator)
-{
-  return (YR_MEMORY_BLOCK*) iterator->context;
-}
-
-
-static YR_MEMORY_BLOCK* _yr_get_next_block(
-    YR_MEMORY_BLOCK_ITERATOR* iterator)
-{
-  return NULL;
-}
-
-
-static const uint8_t* _yr_fetch_block_data(
-    YR_MEMORY_BLOCK* block)
-{
-  return (const uint8_t*) block->context;
-}
-
-
 YR_API int yr_rules_scan_mem(
     YR_RULES* rules,
     const uint8_t* buffer,
@@ -230,27 +202,41 @@ YR_API int yr_rules_scan_mem(
     void* user_data,
     int timeout)
 {
-  YR_MEMORY_BLOCK block;
-  YR_MEMORY_BLOCK_ITERATOR iterator;
-
-  block.size = buffer_size;
-  block.base = 0;
-  block.fetch_data = _yr_fetch_block_data;
-  block.context = (void*) buffer;
-
-  iterator.context = &block;
-  iterator.first = _yr_get_first_block;
-  iterator.next = _yr_get_next_block;
-
-  return yr_rules_scan_mem_blocks(
-      rules,
-      &iterator,
-      flags,
-      callback,
-      user_data,
+  YR_DEBUG_FPRINTF(
+      2,
+      stderr,
+      "+ %s(buffer=%p buffer_size=%zu timeout=%d) {\n",
+      __FUNCTION__,
+      buffer,
+      buffer_size,
       timeout);
-}
 
+  YR_SCANNER* scanner;
+  int result = ERROR_INTERNAL_FATAL_ERROR;
+
+  GOTO_EXIT_ON_ERROR(yr_scanner_create(rules, &scanner));
+
+  yr_scanner_set_callback(scanner, callback, user_data);
+  yr_scanner_set_timeout(scanner, timeout);
+  yr_scanner_set_flags(scanner, flags);
+
+  result = yr_scanner_scan_mem(scanner, buffer, buffer_size);
+
+  yr_scanner_destroy(scanner);
+
+_exit:
+
+  YR_DEBUG_FPRINTF(
+      2,
+      stderr,
+      ""
+      "} = %d AKA %s // %s()\n",
+      result,
+      yr_debug_error_as_string(result),
+      __FUNCTION__);
+
+  return result;
+}
 
 YR_API int yr_rules_scan_file(
     YR_RULES* rules,
@@ -267,13 +253,7 @@ YR_API int yr_rules_scan_file(
   if (result == ERROR_SUCCESS)
   {
     result = yr_rules_scan_mem(
-        rules,
-        mfile.data,
-        mfile.size,
-        flags,
-        callback,
-        user_data,
-        timeout);
+        rules, mfile.data, mfile.size, flags, callback, user_data, timeout);
 
     yr_filemap_unmap(&mfile);
   }
@@ -281,7 +261,6 @@ YR_API int yr_rules_scan_file(
   return result;
 }
 
-
 YR_API int yr_rules_scan_fd(
     YR_RULES* rules,
     YR_FILE_DESCRIPTOR fd,
@@ -297,13 +276,7 @@ YR_API int yr_rules_scan_fd(
   if (result == ERROR_SUCCESS)
   {
     result = yr_rules_scan_mem(
-        rules,
-        mfile.data,
-        mfile.size,
-        flags,
-        callback,
-        user_data,
-        timeout);
+        rules, mfile.data, mfile.size, flags, callback, user_data, timeout);
 
     yr_filemap_unmap_fd(&mfile);
   }
@@ -311,7 +284,6 @@ YR_API int yr_rules_scan_fd(
   return result;
 }
 
-
 YR_API int yr_rules_scan_proc(
     YR_RULES* rules,
     int pid,
@@ -320,11 +292,12 @@ YR_API int yr_rules_scan_proc(
     void* user_data,
     int timeout)
 {
+  YR_DEBUG_FPRINTF(
+      2, stderr, "+ %s(pid=%d timeout=%d) {\n", __FUNCTION__, pid, timeout);
+
   YR_MEMORY_BLOCK_ITERATOR iterator;
 
-  int result = yr_process_open_iterator(
-      pid,
-      &iterator);
+  int result = yr_process_open_iterator(pid, &iterator);
 
   if (result == ERROR_SUCCESS)
   {
@@ -339,13 +312,18 @@ YR_API int yr_rules_scan_proc(
     yr_process_close_iterator(&iterator);
   }
 
+  YR_DEBUG_FPRINTF(
+      2,
+      stderr,
+      "} = %d AKA %s // %s()\n",
+      result,
+      yr_debug_error_as_string(result),
+      __FUNCTION__);
+
   return result;
 }
 
-
-int yr_rules_from_arena(
-    YR_ARENA* arena,
-    YR_RULES** rules)
+int yr_rules_from_arena(YR_ARENA* arena, YR_RULES** rules)
 {
   YR_RULES* new_rules = (YR_RULES*) yr_malloc(sizeof(YR_RULES));
 
@@ -365,13 +343,11 @@ int yr_rules_from_arena(
   new_rules->num_strings = summary->num_strings;
   new_rules->num_namespaces = summary->num_namespaces;
 
-  new_rules->rules_list_head = yr_arena_get_ptr(
-      arena, YR_RULES_TABLE, 0);
+  new_rules->rules_table = yr_arena_get_ptr(arena, YR_RULES_TABLE, 0);
 
-  new_rules->strings_list_head = yr_arena_get_ptr(
-      arena, YR_STRINGS_TABLE, 0);
+  new_rules->strings_table = yr_arena_get_ptr(arena, YR_STRINGS_TABLE, 0);
 
-  new_rules->externals_list_head = yr_arena_get_ptr(
+  new_rules->ext_vars_table = yr_arena_get_ptr(
       arena, YR_EXTERNAL_VARIABLES_TABLE, 0);
 
   new_rules->ac_transition_table = yr_arena_get_ptr(
@@ -383,19 +359,14 @@ int yr_rules_from_arena(
   new_rules->ac_match_pool = yr_arena_get_ptr(
       arena, YR_AC_STATE_MATCHES_POOL, 0);
 
-  new_rules->code_start = yr_arena_get_ptr(
-      arena, YR_CODE_SECTION, 0);
+  new_rules->code_start = yr_arena_get_ptr(arena, YR_CODE_SECTION, 0);
 
   *rules = new_rules;
 
   return ERROR_SUCCESS;
 }
 
-
-
-YR_API int yr_rules_load_stream(
-    YR_STREAM* stream,
-    YR_RULES** rules)
+YR_API int yr_rules_load_stream(YR_STREAM* stream, YR_RULES** rules)
 {
   YR_ARENA* arena;
 
@@ -413,10 +384,7 @@ YR_API int yr_rules_load_stream(
   return ERROR_SUCCESS;
 }
 
-
-YR_API int yr_rules_load(
-    const char* filename,
-    YR_RULES** rules)
+YR_API int yr_rules_load(const char* filename, YR_RULES** rules)
 {
   int result;
 
@@ -435,18 +403,12 @@ YR_API int yr_rules_load(
   return result;
 }
 
-
-YR_API int yr_rules_save_stream(
-    YR_RULES* rules,
-    YR_STREAM* stream)
+YR_API int yr_rules_save_stream(YR_RULES* rules, YR_STREAM* stream)
 {
   return yr_arena_save_stream(rules->arena, stream);
 }
 
-
-YR_API int yr_rules_save(
-    YR_RULES* rules,
-    const char* filename)
+YR_API int yr_rules_save(YR_RULES* rules, const char* filename)
 {
   int result;
 
@@ -465,22 +427,18 @@ YR_API int yr_rules_save(
   return result;
 }
 
-
-static int _uint32_cmp (
-    const void * a,
-    const void * b)
+static int _uint32_cmp(const void* a, const void* b)
 {
-   return (*(uint32_t*) a - *(uint32_t*) b);
+  return (*(uint32_t*) a - *(uint32_t*) b);
 }
 
-YR_API int yr_rules_get_stats(
-    YR_RULES* rules,
-    YR_RULES_STATS *stats)
+YR_API int yr_rules_get_stats(YR_RULES* rules, YR_RULES_STATS* stats)
 {
   memset(stats, 0, sizeof(YR_RULES_STATS));
 
   stats->ac_tables_size = yr_arena_get_current_offset(
-      rules->arena, YR_AC_TRANSITION_TABLE) / sizeof(YR_AC_TRANSITION);
+                              rules->arena, YR_AC_TRANSITION_TABLE) /
+                          sizeof(YR_AC_TRANSITION);
 
   uint32_t* match_list_lengths = (uint32_t*) yr_malloc(
       sizeof(uint32_t) * stats->ac_tables_size);
@@ -494,13 +452,13 @@ YR_API int yr_rules_get_stats(
   float match_list_length_sum = 0;
   int c = 0;
 
-  for (int i = 0; i < stats->ac_tables_size; i++)
+  for (uint32_t i = 0; i < stats->ac_tables_size; i++)
   {
     int match_list_length = 0;
 
     if (rules->ac_match_table[i] != 0)
     {
-      YR_AC_MATCH *m = &rules->ac_match_pool[rules->ac_match_table[i]-1];
+      YR_AC_MATCH* m = &rules->ac_match_pool[rules->ac_match_table[i] - 1];
 
       while (m != NULL)
       {
@@ -534,14 +492,14 @@ YR_API int yr_rules_get_stats(
   for (int i = 0; i < 100; i++)
   {
     if (i < c)
-      stats->top_ac_match_list_lengths[i] = match_list_lengths[c-i-1];
+      stats->top_ac_match_list_lengths[i] = match_list_lengths[c - i - 1];
     else
       stats->top_ac_match_list_lengths[i] = 0;
   }
 
   stats->ac_average_match_list_length = match_list_length_sum / c;
   stats->ac_match_list_length_pctls[0] = match_list_lengths[0];
-  stats->ac_match_list_length_pctls[100] = match_list_lengths[c-1];
+  stats->ac_match_list_length_pctls[100] = match_list_lengths[c - 1];
 
   for (int i = 1; i < 100; i++)
     stats->ac_match_list_length_pctls[i] = match_list_lengths[(c * i) / 100];
@@ -551,11 +509,9 @@ YR_API int yr_rules_get_stats(
   return ERROR_SUCCESS;
 }
 
-
-YR_API int yr_rules_destroy(
-    YR_RULES* rules)
+YR_API int yr_rules_destroy(YR_RULES* rules)
 {
-  YR_EXTERNAL_VARIABLE* external = rules->externals_list_head;
+  YR_EXTERNAL_VARIABLE* external = rules->ext_vars_table;
 
   while (!EXTERNAL_VARIABLE_IS_NULL(external))
   {
@@ -571,8 +527,7 @@ YR_API int yr_rules_destroy(
   return ERROR_SUCCESS;
 }
 
-YR_API void yr_rule_disable(
-    YR_RULE* rule)
+YR_API void yr_rule_disable(YR_RULE* rule)
 {
   YR_STRING* string;
 
@@ -584,9 +539,7 @@ YR_API void yr_rule_disable(
   }
 }
 
-
-YR_API void yr_rule_enable(
-    YR_RULE* rule)
+YR_API void yr_rule_enable(YR_RULE* rule)
 {
   YR_STRING* string;
 
